-------------------------------------------------------------------------------
--	SimpleSystem CPU [DECODER/EXECUTER]										 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/12	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_cpu_decoder is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		-- code
		opecode			: in	std_logic_vector( 15 downto 0 );
		state2			: in	std_logic;
		-- register file
		r0				: in	std_logic_vector( 31 downto 0 );
		r1				: in	std_logic_vector( 31 downto 0 );
		r2				: in	std_logic_vector( 31 downto 0 );
		r3				: in	std_logic_vector( 31 downto 0 );
		r4				: in	std_logic_vector( 31 downto 0 );
		r5				: in	std_logic_vector( 31 downto 0 );
		r6				: in	std_logic_vector( 31 downto 0 );
		r7				: in	std_logic_vector( 31 downto 0 );
		irqret			: in	std_logic_vector( 31 downto 0 );
		-- destination
		set32			: out	std_logic;
		dst_d			: out	std_logic_vector( 31 downto 0 );
		dst_sel			: out	std_logic_vector(  2 downto 0 );
		dst_we			: out	std_logic;
		irqvec_d		: out	std_logic_vector( 31 downto 0 );
		irqvec_we		: out	std_logic;
		irqen_d			: out	std_logic;
		irqen_we		: out	std_logic;
		mem_address		: out	std_logic_vector( 31 downto 0 );
		mem_width		: out	std_logic_vector(  1 downto 0 );
		mem_d			: out	std_logic_vector( 31 downto 0 );
		mem_we			: out	std_logic;
		mem_oe			: out	std_logic;
		state_we		: out	std_logic;
		state_oe		: out	std_logic
	);
end simplesystem_cpu_decoder;

architecture rtl of simplesystem_cpu_decoder is
	signal	ff_set32			: std_logic;
	signal	ff_dst_d			: std_logic_vector( 31 downto 0 );
	signal	ff_dst_sel			: std_logic_vector(  2 downto 0 );
	signal	ff_dst_we			: std_logic;
	signal	ff_irqvec_d			: std_logic_vector( 31 downto 0 );
	signal	ff_irqvec_we		: std_logic;
	signal	ff_irqen_d			: std_logic;
	signal	ff_irqen_we			: std_logic;
	signal	ff_mem_address		: std_logic_vector( 31 downto 0 );
	signal	ff_mem_width		: std_logic_vector(  1 downto 0 );
	signal	ff_mem_d			: std_logic_vector( 31 downto 0 );
	signal	ff_mem_we			: std_logic;
	signal	ff_mem_oe			: std_logic;

	signal	w_en_imm3			: std_logic;
	signal	w_ope				: std_logic_vector(  5 downto 0 );
	signal	w_dst_sel			: std_logic_vector(  2 downto 0 );
	signal	w_src1_sel			: std_logic_vector(  2 downto 0 );
	signal	w_src2_sel			: std_logic_vector(  2 downto 0 );
	signal	w_dst_r				: std_logic_vector( 31 downto 0 );
	signal	w_src1_r			: std_logic_vector( 31 downto 0 );
	signal	w_src2_r			: std_logic_vector( 31 downto 0 );
	signal	w_src2_i			: std_logic_vector( 31 downto 0 );
	signal	w_src2				: std_logic_vector( 31 downto 0 );
	signal	w_imm6				: std_logic_vector(  5 downto 0 );
	signal	w_add				: std_logic_vector( 31 downto 0 );
	signal	w_sub				: std_logic_vector( 31 downto 0 );
	signal	w_mul_tmp			: std_logic_vector( 63 downto 0 );
	signal	w_mul				: std_logic_vector( 31 downto 0 );
	signal	w_lsft_tmp			: std_logic_vector( 31 downto 0 );
	signal	w_lsft				: std_logic_vector( 31 downto 0 );
	signal	w_rsft_tmp			: std_logic_vector( 31 downto 0 );
	signal	w_rsft				: std_logic_vector( 31 downto 0 );
	signal	w_flag				: std_logic_vector( 31 downto 0 );
	signal	w_asft_tmp			: std_logic_vector( 31 downto 0 );
	signal	w_asft				: std_logic_vector( 31 downto 0 );
	signal	w_or				: std_logic_vector( 31 downto 0 );
	signal	w_xor				: std_logic_vector( 31 downto 0 );
	signal	w_and				: std_logic_vector( 31 downto 0 );
	signal	w_ext8				: std_logic_vector( 31 downto 0 );
	signal	w_ext16				: std_logic_vector( 31 downto 0 );
	signal	w_scmp				: std_logic_vector( 31 downto 0 );
	signal	w_cmp				: std_logic_vector( 31 downto 0 );
	signal	w_dst_we			: std_logic;
	signal	w_if_we				: std_logic;
	signal	w_dst_d				: std_logic_vector( 31 downto 0 );
	signal	w_r7p1				: std_logic_vector( 31 downto 0 );
	signal	w_uset6				: std_logic_vector( 31 downto 0 );
	signal	w_sset6				: std_logic_vector( 31 downto 0 );
	signal	w_state_we			: std_logic;
	signal	w_state_oe			: std_logic;
	constant	c_zero			: std_logic_vector( 31 downto 0 ) := (others => '0');
begin

	-- output assignment
	set32		<= ff_set32;
	dst_d		<= ff_dst_d;
	dst_sel		<= ff_dst_sel;
	dst_we		<= ff_dst_we;
	irqvec_d	<= ff_irqvec_d;
	irqvec_we	<= ff_irqvec_we;
	irqen_d		<= ff_irqen_d;
	irqen_we	<= ff_irqen_we;
	mem_address	<= ff_mem_address;
	mem_width	<= ff_mem_width;
	mem_d		<= ff_mem_d;
	mem_we		<= ff_mem_we;
	mem_oe		<= ff_mem_oe;
	state_we	<= w_state_we;
	state_oe	<= w_state_oe;

	w_state_we	<= state2 when( w_ope( 5 downto 2 ) = "1100" )else '0';
	w_state_oe	<= state2 when( w_ope( 5 downto 2 ) = "1000" )else '0';

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_mem_we		<= '0';
			ff_mem_oe		<= '0';
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_mem_we		<= '0';
				ff_mem_oe		<= '0';
			else
				ff_mem_we		<= w_state_we;
				ff_mem_oe		<= w_state_oe;
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_dst_we		<= '0';
			ff_irqvec_we	<= '0';
			ff_irqen_we		<= '0';
			ff_set32		<= '0';
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_dst_we		<= '0';
				ff_irqvec_we	<= '0';
				ff_irqen_we		<= '0';
				ff_set32		<= '0';
			elsif( state2 = '1' )then
				ff_dst_we		<= w_dst_we;
				if( w_ope = conv_std_logic_vector( 15, 6 ) )then
					ff_irqvec_we	<= '1';
				else
					ff_irqvec_we	<= '0';
				end if;
				if( w_ope = conv_std_logic_vector( 14, 6 ) )then
					ff_irqen_we		<= '1';
				else
					ff_irqen_we		<= '0';
				end if;
				if( w_ope = "100011" )then
					ff_set32		<= '1';
				else
					ff_set32		<= '0';
				end if;
			else
				ff_dst_we		<= '0';
				ff_irqvec_we	<= '0';
				ff_irqen_we		<= '0';
				ff_set32		<= '0';
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_dst_d		<= (others => '0');
			ff_dst_sel		<= (others => '0');
			ff_irqvec_d		<= (others => '0');
			ff_irqen_d		<= '0';
			ff_mem_address	<= (others => '0');
			ff_mem_width	<= (others => '0');
			ff_mem_d		<= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_dst_d		<= (others => '0');
				ff_dst_sel		<= (others => '0');
				ff_irqvec_d		<= (others => '0');
				ff_irqen_d		<= '0';
				ff_mem_address	<= (others => '0');
				ff_mem_width	<= (others => '0');
				ff_mem_d		<= (others => '0');
			elsif( state2 = '1' )then
				ff_dst_d		<= w_dst_d;
				ff_dst_sel		<= w_dst_sel;
				ff_irqvec_d		<= w_add;
				ff_irqen_d		<= w_src2(0);
				if( w_ope( 1 downto 0 ) = "11" )then
					ff_mem_address	<= w_r7p1;
				else
					ff_mem_address	<= w_add;
				end if;
				ff_mem_width	<= w_ope( 1 downto 0 );
				ff_mem_d		<= w_dst_r;
			end if;
		end if;
	end process;

	-- opecode decode
	w_en_imm3	<= opecode( 15 );
	w_ope		<= opecode( 14 downto  9 );
	w_dst_sel	<= opecode(  8 downto  6 );
	w_src1_sel	<= opecode(  5 downto  3 );
	w_src2_sel	<= opecode(  2 downto  0 );
	w_imm6		<= opecode(  5 downto  0 );

	with( w_dst_sel )select w_dst_r <=
		r0				when "000",
		r1				when "001",
		r2				when "010",
		r3				when "011",
		r4				when "100",
		r5				when "101",
		r6				when "110",
		r7				when "111",
		(others => 'X')	when others;

	with( w_src1_sel )select w_src1_r <=
		r0				when "000",
		r1				when "001",
		r2				when "010",
		r3				when "011",
		r4				when "100",
		r5				when "101",
		r6				when "110",
		r7				when "111",
		(others => 'X')	when others;

	with( w_src2_sel )select w_src2_r <=
		r0				when "000",
		r1				when "001",
		r2				when "010",
		r3				when "011",
		r4				when "100",
		r5				when "101",
		r6				when "110",
		r7				when "111",
		(others => 'X')	when others;

	w_src2_i	<=	c_zero( 31 downto 3 ) & opecode(  2 downto  0 );
	w_src2		<=	w_src2_r	when( w_en_imm3 = '0' )else
					w_src2_i;

	w_add		<=	w_src1_r + w_src2;
	w_sub		<=	w_src1_r - w_src2;

	w_mul_tmp	<=	w_src1_r * w_src2;
	w_mul		<=	w_mul_tmp( 31 downto 0 );

	with( w_src2( 4 downto 0 ) )select w_lsft_tmp <=
		w_src1_r                                    	when "00000",
		w_src1_r(30 downto 0) & c_zero(31)          	when "00001",
		w_src1_r(29 downto 0) & c_zero(31 downto 30)	when "00010",
		w_src1_r(28 downto 0) & c_zero(31 downto 29)	when "00011",
		w_src1_r(27 downto 0) & c_zero(31 downto 28)	when "00100",
		w_src1_r(26 downto 0) & c_zero(31 downto 27)	when "00101",
		w_src1_r(25 downto 0) & c_zero(31 downto 26)	when "00110",
		w_src1_r(24 downto 0) & c_zero(31 downto 25)	when "00111",
		w_src1_r(23 downto 0) & c_zero(31 downto 24)	when "01000",
		w_src1_r(22 downto 0) & c_zero(31 downto 23)	when "01001",
		w_src1_r(21 downto 0) & c_zero(31 downto 22)	when "01010",
		w_src1_r(20 downto 0) & c_zero(31 downto 21)	when "01011",
		w_src1_r(19 downto 0) & c_zero(31 downto 20)	when "01100",
		w_src1_r(18 downto 0) & c_zero(31 downto 19)	when "01101",
		w_src1_r(17 downto 0) & c_zero(31 downto 18)	when "01110",
		w_src1_r(16 downto 0) & c_zero(31 downto 17)	when "01111",
		w_src1_r(15 downto 0) & c_zero(31 downto 16)	when "10000",
		w_src1_r(14 downto 0) & c_zero(31 downto 15)	when "10001",
		w_src1_r(13 downto 0) & c_zero(31 downto 14)	when "10010",
		w_src1_r(12 downto 0) & c_zero(31 downto 13)	when "10011",
		w_src1_r(11 downto 0) & c_zero(31 downto 12)	when "10100",
		w_src1_r(10 downto 0) & c_zero(31 downto 11)	when "10101",
		w_src1_r( 9 downto 0) & c_zero(31 downto 10)	when "10110",
		w_src1_r( 8 downto 0) & c_zero(31 downto  9)	when "10111",
		w_src1_r( 7 downto 0) & c_zero(31 downto  8)	when "11000",
		w_src1_r( 6 downto 0) & c_zero(31 downto  7)	when "11001",
		w_src1_r( 5 downto 0) & c_zero(31 downto  6)	when "11010",
		w_src1_r( 4 downto 0) & c_zero(31 downto  5)	when "11011",
		w_src1_r( 3 downto 0) & c_zero(31 downto  4)	when "11100",
		w_src1_r( 2 downto 0) & c_zero(31 downto  3)	when "11101",
		w_src1_r( 1 downto 0) & c_zero(31 downto  2)	when "11110",
		w_src1_r( 0 )         & c_zero(31 downto  1)	when "11111",
		(others => '0')                             	when others;

	w_lsft		<=	w_lsft_tmp	when( w_src2( 31 downto 5 ) = c_zero( 31 downto 5 ) )else
					(others => '0');

	with( w_src2( 4 downto 0 ) )select w_rsft_tmp <=
		w_src1_r                                     	when "00000",
		c_zero(31)           & w_src1_r(31 downto  1)	when "00001",
		c_zero(31 downto 30) & w_src1_r(31 downto  2)	when "00010",
		c_zero(31 downto 29) & w_src1_r(31 downto  3)	when "00011",
		c_zero(31 downto 28) & w_src1_r(31 downto  4)	when "00100",
		c_zero(31 downto 27) & w_src1_r(31 downto  5)	when "00101",
		c_zero(31 downto 26) & w_src1_r(31 downto  6)	when "00110",
		c_zero(31 downto 25) & w_src1_r(31 downto  7)	when "00111",
		c_zero(31 downto 24) & w_src1_r(31 downto  8)	when "01000",
		c_zero(31 downto 23) & w_src1_r(31 downto  9)	when "01001",
		c_zero(31 downto 22) & w_src1_r(31 downto 10)	when "01010",
		c_zero(31 downto 21) & w_src1_r(31 downto 11)	when "01011",
		c_zero(31 downto 20) & w_src1_r(31 downto 12)	when "01100",
		c_zero(31 downto 19) & w_src1_r(31 downto 13)	when "01101",
		c_zero(31 downto 18) & w_src1_r(31 downto 14)	when "01110",
		c_zero(31 downto 17) & w_src1_r(31 downto 15)	when "01111",
		c_zero(31 downto 16) & w_src1_r(31 downto 16)	when "10000",
		c_zero(31 downto 15) & w_src1_r(31 downto 17)	when "10001",
		c_zero(31 downto 14) & w_src1_r(31 downto 18)	when "10010",
		c_zero(31 downto 13) & w_src1_r(31 downto 19)	when "10011",
		c_zero(31 downto 12) & w_src1_r(31 downto 20)	when "10100",
		c_zero(31 downto 11) & w_src1_r(31 downto 21)	when "10101",
		c_zero(31 downto 10) & w_src1_r(31 downto 22)	when "10110",
		c_zero(31 downto  9) & w_src1_r(31 downto 23)	when "10111",
		c_zero(31 downto  8) & w_src1_r(31 downto 24)	when "11000",
		c_zero(31 downto  7) & w_src1_r(31 downto 25)	when "11001",
		c_zero(31 downto  6) & w_src1_r(31 downto 26)	when "11010",
		c_zero(31 downto  5) & w_src1_r(31 downto 27)	when "11011",
		c_zero(31 downto  4) & w_src1_r(31 downto 28)	when "11100",
		c_zero(31 downto  3) & w_src1_r(31 downto 29)	when "11101",
		c_zero(31 downto  2) & w_src1_r(31 downto 30)	when "11110",
		c_zero(31 downto  1) & w_src1_r(31)          	when "11111",
		(others => '0')                              	when others;

	w_rsft		<=	w_rsft_tmp	when( w_src2( 31 downto 5 ) = c_zero( 31 downto 5 ) )else
					(others => '0');

	w_flag		<=	(others => w_src1_r(31));

	with( w_src2( 4 downto 0 ) )select w_asft_tmp <=
		w_src1_r                                     	when "00000",
		w_flag(31)           & w_src1_r(31 downto  1)	when "00001",
		w_flag(31 downto 30) & w_src1_r(31 downto  2)	when "00010",
		w_flag(31 downto 29) & w_src1_r(31 downto  3)	when "00011",
		w_flag(31 downto 28) & w_src1_r(31 downto  4)	when "00100",
		w_flag(31 downto 27) & w_src1_r(31 downto  5)	when "00101",
		w_flag(31 downto 26) & w_src1_r(31 downto  6)	when "00110",
		w_flag(31 downto 25) & w_src1_r(31 downto  7)	when "00111",
		w_flag(31 downto 24) & w_src1_r(31 downto  8)	when "01000",
		w_flag(31 downto 23) & w_src1_r(31 downto  9)	when "01001",
		w_flag(31 downto 22) & w_src1_r(31 downto 10)	when "01010",
		w_flag(31 downto 21) & w_src1_r(31 downto 11)	when "01011",
		w_flag(31 downto 20) & w_src1_r(31 downto 12)	when "01100",
		w_flag(31 downto 19) & w_src1_r(31 downto 13)	when "01101",
		w_flag(31 downto 18) & w_src1_r(31 downto 14)	when "01110",
		w_flag(31 downto 17) & w_src1_r(31 downto 15)	when "01111",
		w_flag(31 downto 16) & w_src1_r(31 downto 16)	when "10000",
		w_flag(31 downto 15) & w_src1_r(31 downto 17)	when "10001",
		w_flag(31 downto 14) & w_src1_r(31 downto 18)	when "10010",
		w_flag(31 downto 13) & w_src1_r(31 downto 19)	when "10011",
		w_flag(31 downto 12) & w_src1_r(31 downto 20)	when "10100",
		w_flag(31 downto 11) & w_src1_r(31 downto 21)	when "10101",
		w_flag(31 downto 10) & w_src1_r(31 downto 22)	when "10110",
		w_flag(31 downto  9) & w_src1_r(31 downto 23)	when "10111",
		w_flag(31 downto  8) & w_src1_r(31 downto 24)	when "11000",
		w_flag(31 downto  7) & w_src1_r(31 downto 25)	when "11001",
		w_flag(31 downto  6) & w_src1_r(31 downto 26)	when "11010",
		w_flag(31 downto  5) & w_src1_r(31 downto 27)	when "11011",
		w_flag(31 downto  4) & w_src1_r(31 downto 28)	when "11100",
		w_flag(31 downto  3) & w_src1_r(31 downto 29)	when "11101",
		w_flag(31 downto  2) & w_src1_r(31 downto 30)	when "11110",
		w_flag(31 downto  1) & w_src1_r(31)          	when "11111",
		(others => '0')                              	when others;

	w_asft		<=	w_asft_tmp	when( w_src2( 31 downto 5 ) = c_zero( 31 downto 5 ) )else
					w_flag;

	w_or		<=	w_src1_r or  w_src2;
	w_xor		<=	w_src1_r xor w_src2;
	w_and		<=	w_src1_r and w_src2;

	w_ext8(  31 downto  8 )	<=	(others => w_add(7));
	w_ext8(   7 downto  0 )	<=	w_add(  7 downto 0 );
	w_ext16( 31 downto 16 )	<=	(others => w_add(15));
	w_ext16( 15 downto  0 )	<=	w_add( 15 downto 0 );

	w_cmp		<=	(others => '0')		when( w_src1_r = w_src2 )else
					(others => '1')		when( ('0' & w_src1_r) < ('0' & w_src2) )else
					c_zero( 31 downto 1 ) & '1';

	w_scmp		<=	(others => '0')		when( w_src1_r = w_src2 )else
					(others => '1')		when( w_src1_r < w_src2 )else
					c_zero( 31 downto 1 ) & '1';

	w_if_we		<=	'1'		when( w_ope( 1 downto 0 ) = "00" and w_src1_r  = c_zero )else
					'1'		when( w_ope( 1 downto 0 ) = "01" and w_src1_r /= c_zero )else
					'1'		when( w_ope( 1 downto 0 ) = "10" and w_src1_r(31) = '0' )else
					'1'		when( w_ope( 1 downto 0 ) = "11" and w_src1_r(31) = '1' )else
					'0';

	w_dst_we	<=	'0'		when( w_ope = conv_std_logic_vector( 14, 6 ) )else	--	001110	irqen
					'0'		when( w_ope = conv_std_logic_vector( 15, 6 ) )else	--	001111	irqvec
					w_if_we	when( w_ope( 5 downto 2 ) = "0100" )else			--	0100XX	ifeq, ifneq, ifgte, iflt
					'0'		when( w_ope( 5 ) = '1' )else						--	1XXXXX	load8, load16, load32, set32, store8, store16, store32
					'1';														--	others	add, sub, mul, scmp, lsft, rsft, asft, or, xor, and, cmp, ext8, ext16, uset6, sset6

	w_uset6					<= c_zero( 31 downto 6 ) & w_imm6;
	w_sset6( 31 downto 6 )	<= (others => w_imm6(5));
	w_sset6(  5 downto 0 )	<= w_imm6;

	with( w_ope( 4 downto 0 ) )select w_dst_d <=
		w_add			when "00000",
		w_sub			when "00001",
		w_mul			when "00010",
		w_scmp			when "00011",
		w_lsft			when "00100",
		w_rsft			when "00101",
		w_asft			when "00110",
		w_or			when "00111",
		w_xor			when "01000",
		w_and			when "01001",
		w_cmp			when "01010",
		w_ext8			when "01011",
		w_ext16			when "01100",
		irqret			when "01101",
		w_src2			when "10000",
		w_src2			when "10001",
		w_src2			when "10010",
		w_src2			when "10011",
		w_uset6			when "10100",
		w_sset6			when "10101",
		(others => '-')	when others;

	w_r7p1	<= r7 + 1;

end rtl;
