-------------------------------------------------------------------------------
--	SimpleSystem CPU [STATE MACHINE]										 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/14	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_cpu_state is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		-- state notify
		state0			: out	std_logic;
		state2			: out	std_logic;
		state4			: out	std_logic;
		-- update signal from decoder
		dec_mem_we		: in	std_logic;
		dec_mem_oe		: in	std_logic;
		-- destination
		mem_oe			: out	std_logic;
		mem_ack			: in	std_logic
	);
end simplesystem_cpu_state;

architecture rtl of simplesystem_cpu_state is
	signal ff_state			: std_logic_vector(  2 downto 0 );
	signal w_next_state		: std_logic_vector(  2 downto 0 );
begin

	-- read opecode
	mem_oe			<=	'1'		when( ff_state = "000" )else
						'0';

	-- increment program counter
	state0			<=	'1'		when( ff_state = "000" )else
						'0';
	state2			<=	'1'		when( ff_state = "010" )else
						'0';
	state4			<=	'1'		when( ff_state = "100" )else
						'0';

	-- next state
	w_next_state	<=	"001"	when( ff_state = "000" )else
						"010"	when( ff_state = "001" and mem_ack = '1' )else
						"011"	when( ff_state = "010" and dec_mem_we = '1' )else
						"100"	when( ff_state = "010" and dec_mem_oe = '1' )else
						"101"	when( ff_state = "010" )else
						"000"	when( ff_state = "011" and mem_ack = '1' )else
						"101"	when( ff_state = "100" and mem_ack = '1' )else
						"000"	when( ff_state = "101" )else
						ff_state;

	-- cpu state latch
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_state <= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_state <= (others => '0');
			else
				ff_state <= w_next_state;
			end if;
		end if;
	end process;
end rtl;
