-------------------------------------------------------------------------------
--	testbench for SimpleSystem CPU [MEMORY I/F]								 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/16	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;
	use	work.tb_utility.all;

entity testbench is
end testbench;

architecture bhv of testbench is
	component simplesystem_cpu_decoder
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			-- code
			opecode			: in	std_logic_vector( 15 downto 0 );
			state2			: in	std_logic;
			-- register file
			r0				: in	std_logic_vector( 31 downto 0 );
			r1				: in	std_logic_vector( 31 downto 0 );
			r2				: in	std_logic_vector( 31 downto 0 );
			r3				: in	std_logic_vector( 31 downto 0 );
			r4				: in	std_logic_vector( 31 downto 0 );
			r5				: in	std_logic_vector( 31 downto 0 );
			r6				: in	std_logic_vector( 31 downto 0 );
			r7				: in	std_logic_vector( 31 downto 0 );
			irqret			: in	std_logic_vector( 31 downto 0 );
			-- destination
			set32			: out	std_logic;
			dst_d			: out	std_logic_vector( 31 downto 0 );
			dst_sel			: out	std_logic_vector(  2 downto 0 );
			dst_we			: out	std_logic;
			irqvec_d		: out	std_logic_vector( 31 downto 0 );
			irqvec_we		: out	std_logic;
			irqen_d			: out	std_logic;
			irqen_we		: out	std_logic;
			mem_address		: out	std_logic_vector( 31 downto 0 );
			mem_width		: out	std_logic_vector(  1 downto 0 );
			mem_d			: out	std_logic_vector( 31 downto 0 );
			mem_we			: out	std_logic;
			mem_oe			: out	std_logic;
			state_we		: out	std_logic;
			state_oe		: out	std_logic
		);
	end component;

	type reg_t is array( 0 to 7 ) of std_logic_vector( 31 downto 0 );

	signal reset			: std_logic := '0';
	signal clk				: std_logic;
	signal sreset			: std_logic;
	signal opecode			: std_logic_vector( 15 downto 0 );
	signal state2			: std_logic;
	signal reg				: reg_t;
	signal irqret			: std_logic_vector( 31 downto 0 );
	signal set32			: std_logic;
	signal dst_d			: std_logic_vector( 31 downto 0 );
	signal dst_sel			: std_logic_vector(  2 downto 0 );
	signal dst_we			: std_logic;
	signal irqvec_d			: std_logic_vector( 31 downto 0 );
	signal irqvec_we		: std_logic;
	signal irqen_d			: std_logic;
	signal irqen_we			: std_logic;
	signal mem_address		: std_logic_vector( 31 downto 0 );
	signal mem_width		: std_logic_vector(  1 downto 0 );
	signal mem_d			: std_logic_vector( 31 downto 0 );
	signal mem_we			: std_logic;
	signal mem_oe			: std_logic;
	signal state_we			: std_logic;
	signal state_oe			: std_logic;

	constant op_add			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 0, 6 );
	constant op_sub			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 1, 6 );
	constant op_mul			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 2, 6 );
	constant op_scmp		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 3, 6 );
	constant op_lsft		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 4, 6 );
	constant op_rsft		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 5, 6 );
	constant op_asft		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 6, 6 );
	constant op_or			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 7, 6 );
	constant op_xor			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 8, 6 );
	constant op_and			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector( 9, 6 );
	constant op_cmp			: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(10, 6 );
	constant op_ext8		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(11, 6 );
	constant op_ext16		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(12, 6 );
	constant op_irqret		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(13, 6 );
	constant op_irqen		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(14, 6 );
	constant op_irqvec		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(15, 6 );
	constant op_ifeq		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(16, 6 );
	constant op_ifneq		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(17, 6 );
	constant op_ifgte		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(18, 6 );
	constant op_iflt		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(19, 6 );
	constant op_uset6		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(20, 6 );
	constant op_sset6		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(21, 6 );
	constant op_load8		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(32, 6 );
	constant op_load16		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(33, 6 );
	constant op_load32		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(34, 6 );
	constant op_set32		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(35, 6 );
	constant op_store8		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(48, 6 );
	constant op_store16		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(49, 6 );
	constant op_store32		: std_logic_vector( 5 downto 0 ) := conv_std_logic_vector(50, 6 );

	constant c_all0			: std_logic_vector( 31 downto 0 ) := (others => '0');
	constant c_all1			: std_logic_vector( 31 downto 0 ) := (others => '1');

	signal tb_end			: std_logic := '0';
begin

	u_target: simplesystem_cpu_decoder
	port map(
		reset			=> reset			,
		clk				=> clk				,
		sreset			=> sreset			,
		opecode			=> opecode			,
		state2			=> state2			,
		r0				=> reg(0)			,
		r1				=> reg(1)			,
		r2				=> reg(2)			,
		r3				=> reg(3)			,
		r4				=> reg(4)			,
		r5				=> reg(5)			,
		r6				=> reg(6)			,
		r7				=> reg(7)			,
		irqret			=> irqret			,
		set32			=> set32			,
		dst_d			=> dst_d			,
		dst_sel			=> dst_sel			,
		dst_we			=> dst_we			,
		irqvec_d		=> irqvec_d			,
		irqvec_we		=> irqvec_we		,
		irqen_d			=> irqen_d			,
		irqen_we		=> irqen_we			,
		mem_address		=> mem_address		,
		mem_width		=> mem_width		,
		mem_d			=> mem_d			,
		mem_we			=> mem_we			,
		mem_oe			=> mem_oe			,
		state_we		=> state_we			,
		state_oe		=> state_oe			
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
		variable w_out			: std_logic_vector( 63 downto 0 );
	begin
		sreset			<= '0';
		reg(0)			<= X"12345678";
		reg(1)			<= X"23456781";
		reg(2)			<= X"34567812";
		reg(3)			<= X"45678123";
		reg(4)			<= X"56781234";
		reg(5)			<= X"67812345";
		reg(6)			<= X"78123456";
		reg(7)			<= X"81234567";
		irqret			<= X"87654321";

		wait until( clk'event and clk = '1' );

		state2			<= '1';
		for dst in 0 to 7 loop
			for reg1 in 0 to 7 loop
				for reg2 in 0 to 7 loop
					opecode	<= '0' & op_add		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "add.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg1) + reg(reg2) )										report "add.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "add.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "add.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "add.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "add.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "add.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "add.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_sub		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "sub.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg1) - reg(reg2) )										report "sub.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "sub.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "sub.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "sub.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "sub.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "sub.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "sub.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_mul		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out	:= reg(reg1) * reg(reg2);
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "mul.r (set32)" severity ERROR;
					assert( dst_d		= w_out(31 downto 0) )											report "mul.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "mul.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "mul.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "mul.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "mul.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "mul.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "mul.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_scmp		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "scmp.r (set32)" severity ERROR;
					assert( (dst_d = X"FFFFFFFF" and (signed(reg(reg1)) < signed(reg(reg2)))) or (dst_d = X"00000000" and (reg(reg1) = reg(reg2))) or (dst_d = X"00000001" and (signed(reg(reg1)) > signed(reg(reg2)))) )	report "scmp.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "scmp.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "scmp.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "scmp.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "scmp.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "scmp.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "scmp.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_lsft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "lsft.r (set32)" severity ERROR;
					assert( dst_d		= shift_logical_left( reg(reg1), conv_integer(reg(reg2))) )		report "lsft.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "lsft.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "lsft.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "lsft.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "lsft.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "lsft.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "lsft.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_rsft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "rsft.r (set32)" severity ERROR;
					assert( dst_d		= shift_logical_right( reg(reg1), conv_integer(reg(reg2))) )	report "rsft.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "rsft.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "rsft.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "rsft.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "rsft.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "rsft.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "rsft.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_asft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "asft.r (set32)" severity ERROR;
					assert( dst_d		= shift_arithmetic_right( reg(reg1), conv_integer(reg(reg2))) )	report "asft.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "asft.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "asft.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "asft.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "asft.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "asft.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "asft.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_or		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "or.r (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) or reg(reg2)) )									report "or.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "or.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "or.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "or.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "or.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "or.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "or.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_xor		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "xor.r (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) xor reg(reg2)) )									report "xor.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "xor.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "xor.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "xor.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "xor.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "xor.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "xor.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_and		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "and.r (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) and reg(reg2)) )									report "and.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "and.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "and.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "and.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "and.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "and.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "and.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_cmp		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "cmp.r (set32)" severity ERROR;
					assert( (dst_d = X"FFFFFFFF" and (unsigned(reg(reg1)) < unsigned(reg(reg2)))) or (dst_d = X"00000000" and (reg(reg1) = reg(reg2))) or (dst_d = X"00000001" and (unsigned(reg(reg1)) > unsigned(reg(reg2)))) )	report "cmp.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "cmp.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "cmp.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "cmp.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "cmp.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "cmp.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "cmp.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_ext8		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					w_out( 31 downto 0 ) := reg(reg1) + reg(reg2);
					assert( set32		= '0' )															report "ext8.r (set32)" severity ERROR;
					assert( (w_out(7) = '1' and dst_d = (c_all1(31 downto 8) & w_out(7 downto 0))) or (w_out(7) = '0' and dst_d = (c_all0(31 downto 8) & w_out(7 downto 0))) )	report "ext8.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "ext8.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "ext8.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ext8.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ext8.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ext8.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ext8.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_ext16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					w_out( 31 downto 0 ) := reg(reg1) + reg(reg2);
					assert( set32		= '0' )															report "ext16.r (set32)" severity ERROR;
					assert( (w_out(15) = '1' and dst_d = (c_all1(31 downto 16) & w_out(15 downto 0))) or (w_out(15) = '0' and dst_d = (c_all0(31 downto 16) & w_out(15 downto 0))) )	report "ext16.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "ext16.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "ext16.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ext16.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ext16.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ext16.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ext16.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_irqret	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqret.r (set32)" severity ERROR;
					assert( dst_d		= irqret )														report "irqret.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "irqret.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "irqret.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "irqret.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "irqret.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqret.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqret.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_irqen	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqen.r (set32)" severity ERROR;
					assert( dst_we		= '0' )															report "irqen.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "irqen.r (irqvec_we)" severity ERROR;
					assert( irqen_d		= reg(reg2)(0) )												report "irqen.r (irqen_d)" severity ERROR;
					assert( irqen_we	= '1' )															report "irqen.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqen.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqen.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_irqvec	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqvec.r (set32)" severity ERROR;
					assert( dst_we		= '0' )															report "irqvec.r (dst_we)" severity ERROR;
					assert( irqvec_d	= (reg(reg1) + reg(reg2)) )										report "irqvec.r (irqvec_d)" severity ERROR;
					assert( irqvec_we	= '1' )															report "irqvec.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "irqvec.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqvec.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqvec.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_ifeq	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifeq.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg2) )													report "ifeq.r (dst_d)" severity ERROR;
					assert( ((reg(reg1) = c_all0) and (dst_we = '1')) or ((reg(reg1) /= c_all0) and (dst_we = '0')) )	report "ifeq.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifeq.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifeq.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifeq.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifeq.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_ifneq	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifneq.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg2) )													report "ifneq.r (dst_d)" severity ERROR;
					assert( ((reg(reg1) = c_all0) and (dst_we = '0')) or ((reg(reg1) /= c_all0) and (dst_we = '1')) )	report "ifneq.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifneq.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifneq.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifneq.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifneq.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_ifgte	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifgte.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg2) )													report "ifgte.r (dst_d)" severity ERROR;
					assert( ((reg(reg1)(31) = '0') and (dst_we = '1')) or ((reg(reg1)(31) = '1') and (dst_we = '0')) )	report "ifgte.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifgte.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifgte.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifgte.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifgte.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_iflt	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "iflt.r (set32)" severity ERROR;
					assert( dst_d		= reg(reg2) )													report "iflt.r (dst_d)" severity ERROR;
					assert( ((reg(reg1)(31) = '1') and (dst_we = '1')) or ((reg(reg1)(31) = '0') and (dst_we = '0')) )	report "iflt.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "iflt.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "iflt.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "iflt.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "iflt.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_uset6	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out := conv_std_logic_vector( reg1 * 8 + reg2, 64 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "uset6.r (set32)" severity ERROR;
					assert( dst_d = w_out(31 downto 0) )												report "uset6.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "uset6.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "uset6.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "uset6.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "uset6.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "uset6.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "uset6.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_sset6	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out(31 downto 6) := (others => w_out(5));
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "sset6.r (set32)" severity ERROR;
					assert( dst_d = w_out(31 downto 0) )												report "sset6.r (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "sset6.r (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "sset6.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "sset6.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "sset6.r (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "sset6.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "sset6.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_load8	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load8.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load8.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load8.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load8.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load8.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "load8.r (mem_address)" severity ERROR;
					assert( mem_width	= "00" )														report "load8.r (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load8.r (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load8.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_load16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load16.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load16.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load16.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load16.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load16.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "load16.r (mem_address)" severity ERROR;
					assert( mem_width	= "01" )														report "load16.r (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load16.r (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load16.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_load32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load32.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load32.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load32.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load32.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load32.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "load32.r (mem_address)" severity ERROR;
					assert( mem_width	= "10" )														report "load32.r (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load32.r (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load32.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_set32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '1' )															report "set32.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "set32.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "set32.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "set32.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "set32.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(7) + 1) )												report "set32.r (mem_address)" severity ERROR;
					assert( mem_width	= "11" )														report "set32.r (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "set32.r (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "set32.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_store8	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store8.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store8.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store8.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store8.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store8.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "store8.r (mem_address)" severity ERROR;
					assert( mem_width	= "00" )														report "store8.r (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store8.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store8.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_store16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store16.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store16.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store16.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store16.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store16.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "store16.r (mem_address)" severity ERROR;
					assert( mem_width	= "01" )														report "store16.r (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store16.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store16.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '0' & op_store32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store32.r (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store32.r (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store32.r (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store32.r (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store32.r (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg(reg2)) )										report "store32.r (mem_address)" severity ERROR;
					assert( mem_width	= "10" )														report "store32.r (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store32.r (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store32.r (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );
				end loop;
			end loop;
		end loop;

		for dst in 0 to 7 loop
			for reg1 in 0 to 7 loop
				for reg2 in 0 to 7 loop
					opecode	<= '1' & op_add		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "add.i (set32)" severity ERROR;
					assert( dst_d		= reg(reg1) + reg2 )											report "add.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "add.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "add.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "add.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "add.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "add.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "add.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_sub		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "sub.i (set32)" severity ERROR;
					assert( dst_d		= reg(reg1) - reg2 )											report "sub.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "sub.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "sub.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "sub.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "sub.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "sub.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "sub.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_mul		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out	:= reg(reg1) * (c_all0(31 downto 3) & conv_std_logic_vector( reg2, 3 ));
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "mul.i (set32)" severity ERROR;
					assert( dst_d		= w_out(31 downto 0) )											report "mul.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "mul.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "mul.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "mul.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "mul.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "mul.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "mul.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_scmp		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "scmp.i (set32)" severity ERROR;
					assert( (dst_d = X"FFFFFFFF" and (signed(reg(reg1)) < signed(conv_std_logic_vector( reg2, 3 )))) or (dst_d = X"00000000" and (reg(reg1) = conv_std_logic_vector( reg2, 3 ))) or (dst_d = X"00000001" and (signed(reg(reg1)) > signed(conv_std_logic_vector( reg2, 3 )))) )	report "scmp.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "scmp.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "scmp.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "scmp.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "scmp.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "scmp.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "scmp.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_lsft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "lsft.i (set32)" severity ERROR;
					assert( dst_d		= shift_logical_left( reg(reg1), reg2) )						report "lsft.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "lsft.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "lsft.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "lsft.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "lsft.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "lsft.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "lsft.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_rsft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "rsft.i (set32)" severity ERROR;
					assert( dst_d		= shift_logical_right( reg(reg1), reg2) )						report "rsft.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "rsft.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "rsft.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "rsft.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "rsft.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "rsft.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "rsft.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_asft		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "asft.i (set32)" severity ERROR;
					assert( dst_d		= shift_arithmetic_right( reg(reg1), reg2) )					report "asft.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "asft.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "asft.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "asft.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "asft.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "asft.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "asft.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_or		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "or.i (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) or (c_all0( 31 downto 3 ) & conv_std_logic_vector( reg2, 3 ))) )	report "or.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "or.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "or.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "or.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "or.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "or.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "or.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_xor		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "xor.i (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) xor (c_all0( 31 downto 3 ) & conv_std_logic_vector( reg2, 3 ))) )	report "xor.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "xor.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "xor.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "xor.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "xor.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "xor.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "xor.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_and		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "and.i (set32)" severity ERROR;
					assert( dst_d		= (reg(reg1) and (c_all0( 31 downto 3 ) & conv_std_logic_vector( reg2, 3 ))) )	report "and.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "and.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "and.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "and.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "and.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "and.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "and.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_cmp		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "cmp.i (set32)" severity ERROR;
					assert( (dst_d = X"FFFFFFFF" and (unsigned(reg(reg1)) < unsigned(conv_std_logic_vector( reg2, 3 )))) or (dst_d = X"00000000" and (reg(reg1) = conv_std_logic_vector( reg2, 3 ))) or (dst_d = X"00000001" and (unsigned(reg(reg1)) > unsigned(conv_std_logic_vector( reg2, 3 )))) )	report "cmp.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "cmp.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "cmp.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "cmp.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "cmp.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "cmp.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "cmp.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_ext8		& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					w_out( 31 downto 0 ) := reg(reg1) + reg2;
					assert( set32		= '0' )															report "ext8.i (set32)" severity ERROR;
					assert( (w_out(7) = '1' and dst_d = (c_all1(31 downto 8) & w_out(7 downto 0))) or (w_out(7) = '0' and dst_d = (c_all0(31 downto 8) & w_out(7 downto 0))) )	report "ext8.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "ext8.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "ext8.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ext8.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ext8.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ext8.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ext8.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_ext16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					w_out( 31 downto 0 ) := reg(reg1) + reg2;
					assert( set32		= '0' )															report "ext16.i (set32)" severity ERROR;
					assert( (w_out(15) = '1' and dst_d = (c_all1(31 downto 16) & w_out(15 downto 0))) or (w_out(15) = '0' and dst_d = (c_all0(31 downto 16) & w_out(15 downto 0))) )	report "ext16.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "ext16.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "ext16.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ext16.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ext16.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ext16.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ext16.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_irqret	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqret.i (set32)" severity ERROR;
					assert( dst_d		= irqret )														report "irqret.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "irqret.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "irqret.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "irqret.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "irqret.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqret.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqret.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_irqen	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqen.i (set32)" severity ERROR;
					assert( dst_we		= '0' )															report "irqen.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "irqen.i (irqvec_we)" severity ERROR;
					assert( irqen_d		= conv_std_logic_vector( reg2, 3 )(0) )							report "irqen.i (irqen_d)" severity ERROR;
					assert( irqen_we	= '1' )															report "irqen.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqen.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqen.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_irqvec	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "irqvec.i (set32)" severity ERROR;
					assert( dst_we		= '0' )															report "irqvec.i (dst_we)" severity ERROR;
					assert( irqvec_d	= (reg(reg1) + reg2) )											report "irqvec.i (irqvec_d)" severity ERROR;
					assert( irqvec_we	= '1' )															report "irqvec.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "irqvec.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "irqvec.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "irqvec.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_ifeq	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifeq.i (set32)" severity ERROR;
					assert( dst_d		= reg2 )														report "ifeq.i (dst_d)" severity ERROR;
					assert( ((reg(reg1) = c_all0) and (dst_we = '1')) or ((reg(reg1) /= c_all0) and (dst_we = '0')) )	report "ifeq.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifeq.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifeq.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifeq.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifeq.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_ifneq	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifneq.i (set32)" severity ERROR;
					assert( dst_d		= reg2 )														report "ifneq.i (dst_d)" severity ERROR;
					assert( ((reg(reg1) = c_all0) and (dst_we = '0')) or ((reg(reg1) /= c_all0) and (dst_we = '1')) )	report "ifneq.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifneq.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifneq.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifneq.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifneq.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_ifgte	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "ifgte.i (set32)" severity ERROR;
					assert( dst_d		= reg2 )														report "ifgte.i (dst_d)" severity ERROR;
					assert( ((reg(reg1)(31) = '0') and (dst_we = '1')) or ((reg(reg1)(31) = '1') and (dst_we = '0')) )	report "ifgte.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "ifgte.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "ifgte.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "ifgte.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "ifgte.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_iflt	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "iflt.i (set32)" severity ERROR;
					assert( dst_d		= reg2 )														report "iflt.i (dst_d)" severity ERROR;
					assert( ((reg(reg1)(31) = '1') and (dst_we = '1')) or ((reg(reg1)(31) = '0') and (dst_we = '0')) )	report "iflt.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "iflt.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "iflt.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "iflt.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "iflt.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_uset6	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out := conv_std_logic_vector( reg1 * 8 + reg2, 64 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "uset6.i (set32)" severity ERROR;
					assert( dst_d = w_out(31 downto 0) )												report "uset6.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "uset6.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "uset6.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "uset6.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "uset6.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "uset6.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "uset6.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_sset6	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					w_out(31 downto 6) := (others => w_out(5));
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "sset6.i (set32)" severity ERROR;
					assert( dst_d = w_out(31 downto 0) )												report "sset6.i (dst_d)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "sset6.i (dst_sel)" severity ERROR;
					assert( dst_we		= '1' )															report "sset6.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "sset6.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "sset6.i (irqen_we)" severity ERROR;
					assert( mem_we		= '0' )															report "sset6.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "sset6.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_load8	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load8.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load8.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load8.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load8.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load8.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "load8.i (mem_address)" severity ERROR;
					assert( mem_width	= "00" )														report "load8.i (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load8.i (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load8.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_load16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load16.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load16.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load16.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load16.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load16.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "load16.i (mem_address)" severity ERROR;
					assert( mem_width	= "01" )														report "load16.i (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load16.i (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load16.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_load32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "load32.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "load32.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "load32.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "load32.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "load32.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "load32.i (mem_address)" severity ERROR;
					assert( mem_width	= "10" )														report "load32.i (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "load32.i (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "load32.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_set32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '1' )															report "set32.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "set32.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "set32.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "set32.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "set32.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(7) + 1) )												report "set32.i (mem_address)" severity ERROR;
					assert( mem_width	= "11" )														report "set32.i (mem_width)" severity ERROR;
					assert( mem_we		= '0' )															report "set32.i (mem_we)" severity ERROR;
					assert( mem_oe		= '1' )															report "set32.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_store8	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store8.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store8.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store8.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store8.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store8.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "store8.i (mem_address)" severity ERROR;
					assert( mem_width	= "00" )														report "store8.i (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store8.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store8.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_store16	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store16.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store16.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store16.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store16.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store16.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "store16.i (mem_address)" severity ERROR;
					assert( mem_width	= "01" )														report "store16.i (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store16.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store16.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );

					opecode	<= '1' & op_store32	& conv_std_logic_vector( dst, 3 ) & conv_std_logic_vector( reg1, 3 ) & conv_std_logic_vector( reg2, 3 );
					wait until( clk'event and clk = '1' );
					wait until( clk'event and clk = '1' );
					assert( set32		= '0' )															report "store32.i (set32)" severity ERROR;
					assert( dst_sel		= conv_std_logic_vector( dst, 3 ) )								report "store32.i (dst_sel)" severity ERROR;
					assert( dst_we		= '0' )															report "store32.i (dst_we)" severity ERROR;
					assert( irqvec_we	= '0' )															report "store32.i (irqvec_we)" severity ERROR;
					assert( irqen_we	= '0' )															report "store32.i (irqen_we)" severity ERROR;
					assert( mem_address	= (reg(reg1) + reg2) )											report "store32.i (mem_address)" severity ERROR;
					assert( mem_width	= "10" )														report "store32.i (mem_width)" severity ERROR;
					assert( mem_we		= '1' )															report "store32.i (mem_we)" severity ERROR;
					assert( mem_oe		= '0' )															report "store32.i (mem_oe)" severity ERROR;
					wait until( clk'event and clk = '1' );
				end loop;
			end loop;
		end loop;

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		tb_end <= '1';
		wait;
	end process;
end bhv;
