-------------------------------------------------------------------------------
--	utility functions for testbench											 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/09/02	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

package tb_utility is
	---------------------------------------------------------------------------
	function shift_logical_left(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector;

	---------------------------------------------------------------------------
	function shift_logical_right(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector;

	---------------------------------------------------------------------------
	function shift_arithmetic_right(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector;

end tb_utility;

package body tb_utility is

	---------------------------------------------------------------------------
	function shift_logical_left(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector is
		variable return_value		: std_logic_vector( data'high downto 0 );
		constant c_zero				: std_logic_vector( data'high downto 0 ) := (others => '0');
	begin
		if( (bits < 0) or (bits >= data'high) )then
			return_value :=	c_zero;
		elsif( bits = 0 )then
			return_value :=	data;
		else
			return_value :=	data( data'high-bits downto 0 ) & c_zero( bits-1 downto 0 );
		end if;
		return return_value;
	end shift_logical_left;

	---------------------------------------------------------------------------
	function shift_logical_right(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector is
		variable return_value		: std_logic_vector( data'high downto 0 );
		constant c_zero				: std_logic_vector( data'high downto 0 ) := (others => '0');
	begin
		if( (bits < 0) or (bits >= data'high) )then
			return_value :=	c_zero;
		elsif( bits = 0 )then
			return_value :=	data;
		else
			return_value :=	c_zero( bits-1 downto 0 ) & data( data'high downto bits );
		end if;
		return return_value;
	end shift_logical_right;

	---------------------------------------------------------------------------
	function shift_arithmetic_right(
		data		: std_logic_vector;
		bits		: integer
	)	return std_logic_vector is
		variable return_value		: std_logic_vector( data'high downto 0 );
		variable data_flag			: std_logic_vector( data'high downto 0 );
		constant c_zero				: std_logic_vector( data'high downto 0 ) := (others => '0');
	begin
		data_flag :=	(others => data( data'high ) );
		if( (bits < 0) or (bits >= data'high) )then
			return_value :=	data_flag;
		elsif( bits = 0 )then
			return_value :=	data;
		else
			return_value :=	data_flag( bits-1 downto 0 ) & data( data'high downto bits );
		end if;
		return return_value;
	end shift_arithmetic_right;

end tb_utility;
