-------------------------------------------------------------------------------
--	testbench for SimpleSystem CPU [MEMORY I/F]								 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/16	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity testbench is
end testbench;

architecture bhv of testbench is
	component simplesystem_cpu_memory
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			-- request from state
			fetch_address	: in	std_logic_vector( 31 downto 0 );
			fetch_oe		: in	std_logic;
			fetch_q			: out	std_logic_vector( 15 downto 0 );
			fetch_q_en		: out	std_logic;
			-- request from decoder
			dec_address		: in	std_logic_vector( 31 downto 0 );
			dec_d			: in	std_logic_vector( 31 downto 0 );
			dec_width		: in	std_logic_vector(  1 downto 0 );	-- "00": 1byte, "01": 2byte, "1-": 4byte
			dec_we			: in	std_logic;
			dec_oe			: in	std_logic;
			dec_q			: out	std_logic_vector( 31 downto 0 );
			dec_q_en		: out	std_logic;
			-- request to 16bit bus interface
			cpu_address		: out	std_logic_vector( 30 downto 0 );
			cpu_d			: out	std_logic_vector( 15 downto 0 );
			cpu_we0			: out	std_logic;							-- lower byte
			cpu_we1			: out	std_logic;							-- upper byte
			cpu_oe			: out	std_logic;
			cpu_q			: in	std_logic_vector( 15 downto 0 );
			cpu_q_en		: in	std_logic
		);
	end component;

	signal reset			: std_logic;
	signal clk				: std_logic;
	signal sreset			: std_logic;
	signal fetch_address	: std_logic_vector( 31 downto 0 );
	signal fetch_oe			: std_logic;
	signal fetch_q			: std_logic_vector( 15 downto 0 );
	signal fetch_q_en		: std_logic;
	signal dec_address		: std_logic_vector( 31 downto 0 );
	signal dec_d			: std_logic_vector( 31 downto 0 );
	signal dec_width		: std_logic_vector(  1 downto 0 );
	signal dec_we			: std_logic;
	signal dec_oe			: std_logic;
	signal dec_q			: std_logic_vector( 31 downto 0 );
	signal dec_q_en			: std_logic;
	signal cpu_address		: std_logic_vector( 30 downto 0 );
	signal cpu_d			: std_logic_vector( 15 downto 0 );
	signal cpu_we0			: std_logic;
	signal cpu_we1			: std_logic;
	signal cpu_oe			: std_logic;
	signal cpu_q			: std_logic_vector( 15 downto 0 );
	signal cpu_q_en			: std_logic;
	signal cpu_address0		: std_logic_vector( 31 downto 0 );

	signal test_number		: integer := 0;
	signal tb_end			: std_logic := '0';
begin

	u_target: simplesystem_cpu_memory
	port map(
		reset			=> reset			,
		clk				=> clk				,
		sreset			=> sreset			,
		fetch_address	=> fetch_address	,
		fetch_oe		=> fetch_oe			,
		fetch_q			=> fetch_q			,
		fetch_q_en		=> fetch_q_en		,
		dec_address		=> dec_address		,
		dec_d			=> dec_d			,
		dec_width		=> dec_width		,
		dec_we			=> dec_we			,
		dec_oe			=> dec_oe			,
		dec_q			=> dec_q			,
		dec_q_en		=> dec_q_en			,
		cpu_address		=> cpu_address		,
		cpu_d			=> cpu_d			,
		cpu_we0			=> cpu_we0			,
		cpu_we1			=> cpu_we1			,
		cpu_oe			=> cpu_oe			,
		cpu_q			=> cpu_q			,
		cpu_q_en		=> cpu_q_en			
	);
	cpu_address0 <= cpu_address & '0';

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
	begin
		-------------------------------------------------------
		-- reset
		test_number		<= 0;
		reset			<= '1';
		sreset			<= '0';
		fetch_address	<= conv_std_logic_vector( 12345678, 32 );
		fetch_oe		<= '0';
		dec_address		<= conv_std_logic_vector( 23456789, 32 );
		dec_d			<= X"34567890";
		dec_width		<= "00";
		dec_we			<= '0';
		dec_oe			<= '0';
		cpu_q			<= X"4567";
		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );

		reset			<= '0';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from fetch
		test_number		<= 1;
		fetch_address	<= conv_std_logic_vector( 22334455, 32 );
		fetch_oe		<= '1';
		wait until( clk'event and clk = '1' );
		fetch_address	<= conv_std_logic_vector( 12345678, 32 );
		fetch_oe		<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= X"4567";
		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from decoder for 8bit lower byte
		test_number		<= 2;
		dec_address		<= conv_std_logic_vector( 22334454, 32 );
		dec_width		<= "00";
		dec_oe			<= '1';
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_oe			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= X"4567";
		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from decoder for 8bit upper byte
		test_number		<= 3;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "00";
		dec_oe			<= '1';
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_oe			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= X"4567";
		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from decoder for 16bit
		test_number		<= 4;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "01";
		dec_oe			<= '1';
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_oe			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= X"4567";
		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from decoder for 32bit
		test_number		<= 5;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "10";
		dec_oe			<= '1';
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_oe			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 7788, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- read request from decoder for 32bit
		test_number		<= 6;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "11";
		dec_oe			<= '1';
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_oe			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 5566, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q			<= conv_std_logic_vector( 7788, 16 );
		cpu_q_en		<= '1';
		wait until( clk'event and clk = '1' );

		cpu_q_en		<= '0';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- write request from decoder for 8bit lower byte
		test_number		<= 7;
		dec_address		<= conv_std_logic_vector( 22334454, 32 );
		dec_width		<= "00";
		dec_we			<= '1';
		dec_d			<= X"12345678";
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_we			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- write request from decoder for 8bit upper byte
		test_number		<= 8;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "00";
		dec_we			<= '1';
		dec_d			<= X"12345678";
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_we			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- write request from decoder for 16bit
		test_number		<= 9;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "01";
		dec_we			<= '1';
		dec_d			<= X"12345678";
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_we			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- write request from decoder for 32bit
		test_number		<= 10;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "10";
		dec_we			<= '1';
		dec_d			<= X"12345678";
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_we			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-------------------------------------------------------
		-- write request from decoder for 32bit
		test_number		<= 11;
		dec_address		<= conv_std_logic_vector( 22334455, 32 );
		dec_width		<= "11";
		dec_we			<= '1';
		dec_d			<= X"12345678";
		wait until( clk'event and clk = '1' );
		dec_address		<= conv_std_logic_vector( 12345678, 32 );
		dec_we			<= '0';
		wait until( clk'event and clk = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		test_number		<= 999;
		reset			<= '0';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		tb_end <= '1';
		wait;
	end process;
end bhv;
