-------------------------------------------------------------------------------
--	testbench for SimpleSystem CPU [STATE MACHINE]							 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/15	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity testbench is
end testbench;

architecture bhv of testbench is
	component simplesystem_cpu_state
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			-- state signal
			state0			: out	std_logic;
			state2			: out	std_logic;
			state4			: out	std_logic;
			-- update signal from decoder
			dec_mem_we		: in	std_logic;
			dec_mem_oe		: in	std_logic;
			-- destination
			mem_oe			: out	std_logic;
			mem_ack			: in	std_logic
		);
	end component;

	signal reset			: std_logic;
	signal clk				: std_logic;
	signal sreset			: std_logic;
	signal state0			: std_logic;
	signal state2			: std_logic;
	signal state4			: std_logic;
	signal dec_mem_we		: std_logic;
	signal dec_mem_oe		: std_logic;
	signal mem_oe			: std_logic;
	signal mem_ack			: std_logic;

	signal test_number		: integer := 0;
	signal tb_end			: std_logic := '0';
begin

	u_target: simplesystem_cpu_state
	port map(
		reset			=> reset			,
		clk				=> clk				,
		sreset			=> sreset			,
		state0			=> state0			,
		state2			=> state2			,
		state4			=> state4			,
		dec_mem_we		=> dec_mem_we		,
		dec_mem_oe		=> dec_mem_oe		,
		mem_oe			=> mem_oe			,
		mem_ack			=> mem_ack			
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
	begin
		-------------------------------------------------------
		-- reset
		test_number		<= 0;
		reset			<= '1';
		sreset			<= '0';
		dec_mem_we		<= '0';
		dec_mem_oe		<= '0';
		mem_ack			<= '0';
		for i in 0 to 9 loop
			wait until( clk'event and clk = '1' );
		end loop;

		-- 0 -> 1 -> 2 -> 5
		test_number		<= 1;
		reset			<= '0';
		wait until( clk'event and clk = '1' );		-- state 0
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '1';
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '0';
		wait until( clk'event and clk = '1' );		-- state 2
		wait until( clk'event and clk = '1' );		-- state 5

		-- 0 -> 1 -> 2 -> 3
		test_number		<= 2;
		wait until( clk'event and clk = '1' );		-- state 0
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '1';
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '0';
		dec_mem_we		<= '1';
		wait until( clk'event and clk = '1' );		-- state 2
		dec_mem_we		<= '0';
		wait until( clk'event and clk = '1' );		-- state 3
		wait until( clk'event and clk = '1' );		-- state 3
		wait until( clk'event and clk = '1' );		-- state 3
		wait until( clk'event and clk = '1' );		-- state 3
		wait until( clk'event and clk = '1' );		-- state 3
		mem_ack			<= '1';
		wait until( clk'event and clk = '1' );		-- state 3
		mem_ack			<= '0';

		-- 0 -> 1 -> 2 -> 4 -> 5
		test_number		<= 2;
		wait until( clk'event and clk = '1' );		-- state 0
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '1';
		wait until( clk'event and clk = '1' );		-- state 1
		mem_ack			<= '0';
		dec_mem_oe		<= '1';
		wait until( clk'event and clk = '1' );		-- state 2
		dec_mem_oe		<= '0';
		wait until( clk'event and clk = '1' );		-- state 4
		wait until( clk'event and clk = '1' );		-- state 4
		mem_ack			<= '1';
		wait until( clk'event and clk = '1' );		-- state 4
		mem_ack			<= '0';
		wait until( clk'event and clk = '1' );		-- state 5

		test_number		<= 3;
		reset			<= '1';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		tb_end <= '1';
		wait;
	end process;
end bhv;
