-------------------------------------------------------------------------------
--	testbench for SimpleSystem CPU [STATE MACHINE]							 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/15	t.hara	first release										 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity testbench is
end testbench;

architecture bhv of testbench is
	component simplesystem_cpu
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			-- interrupt request
			irq				: in	std_logic;
			-- bus interface
			cpu_address		: out	std_logic_vector( 30 downto 0 );
			cpu_d			: out	std_logic_vector( 15 downto 0 );
			cpu_we0			: out	std_logic;							-- lower byte
			cpu_we1			: out	std_logic;							-- upper byte
			cpu_oe			: out	std_logic;
			cpu_q			: in	std_logic_vector( 15 downto 0 );
			cpu_q_en		: in	std_logic
		);
	end component;

	signal reset			: std_logic;
	signal clk				: std_logic;
	signal irq				: std_logic;
	signal cpu_address		: std_logic_vector( 30 downto 0 );
	signal cpu_d			: std_logic_vector( 15 downto 0 );
	signal cpu_we0			: std_logic;							-- lower byte
	signal cpu_we1			: std_logic;							-- upper byte
	signal cpu_oe			: std_logic;
	signal cpu_q			: std_logic_vector( 15 downto 0 );
	signal cpu_q_en			: std_logic;

	signal test_number		: integer := 0;
	signal tb_end			: std_logic := '0';
begin

	u_target: simplesystem_cpu
	port map(
		reset			=> reset			,
		clk				=> clk				,
		irq				=> irq				,
		cpu_address		=> cpu_address		,
		cpu_d			=> cpu_d			,
		cpu_we0			=> cpu_we0			,
		cpu_we1			=> cpu_we1			,
		cpu_oe			=> cpu_oe			,
		cpu_q			=> cpu_q			,
		cpu_q_en		=> cpu_q_en			
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
	begin
		-------------------------------------------------------
		-- reset
		test_number		<= 0;
		reset			<= '1';
		irq				<= '0';
		cpu_q			<= (others => '0');
		cpu_q_en		<= '0';
		for i in 0 to 9 loop
			wait until( clk'event and clk = '1' );
		end loop;

		test_number		<= 999;
		reset			<= '0';
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		tb_end <= '1';
		wait;
	end process;
end bhv;
