-------------------------------------------------------------------------------
--	SimpleSystem parallel I/O												 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/22	t.hara	first release										 --
-------------------------------------------------------------------------------
--	Address																	 --
--	0	rw	set 16bit for output data										 --
--	1	r	read 16bit from input data										 --
-------------------------------------------------------------------------------
library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_pio is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		pio_a			: in	std_logic;
		pio_d			: in	std_logic_vector( 15 downto 0 );
		pio_oe			: in	std_logic;
		pio_we0			: in	std_logic;
		pio_we1			: in	std_logic;
		pio_q			: out	std_logic_vector( 15 downto 0 );
		pio_q_en		: out	std_logic;
		pio_in			: in	std_logic_vector( 15 downto 0 );
		pio_out			: out	std_logic_vector( 15 downto 0 )
	);
end simplesystem_pio;

architecture rtl of simplesystem_pio is
	signal w_q			: std_logic_vector( 15 downto 0 );
	signal ff_q			: std_logic_vector( 15 downto 0 );
	signal ff_q_en		: std_logic;
	signal ff_out		: std_logic_vector( 15 downto 0 );
begin

	-- output assignment
	pio_out		<= ff_out;
	pio_q		<= ff_q;
	pio_q_en	<= ff_q_en;

	-- output signal
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_out(  7 downto  0 ) <= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_out(  7 downto  0 ) <= (others => '0');
			elsif( pio_a = '0' and pio_we0 = '1' )then
				ff_out(  7 downto  0 ) <= pio_d(  7 downto 0 );
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_out( 15 downto  8 ) <= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_out( 15 downto  8 ) <= (others => '0');
			elsif( pio_a = '0' and pio_we1 = '1' )then
				ff_out( 15 downto  8 ) <= pio_d( 15 downto 8 );
			end if;
		end if;
	end process;

	-- read back
	w_q <=	ff_out	when( pio_a = '0' )else
			pio_in;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_q	<= (others => '0');
			ff_q_en	<= '0';
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_q	<= (others => '0');
				ff_q_en	<= '0';
			elsif( (pio_oe or pio_we0 or pio_we1) = '1' )then
				ff_q	<= w_q;
				ff_q_en	<= '1';
			else
				ff_q	<= (others => '0');
				ff_q_en	<= '0';
			end if;
		end if;
	end process;
end rtl;
