-------------------------------------------------------------------------------
--	smallsystem RAM															 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/20	t.hara														 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_ram is
	generic(
		end_address_bit	: in	integer := 5		-- (2^n - 1)
	);
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		ram_a			: in	std_logic_vector( end_address_bit-1 downto 0 );
		ram_d			: in	std_logic_vector( 15 downto 0 );
		ram_oe			: in	std_logic;
		ram_we0			: in	std_logic;
		ram_we1			: in	std_logic;
		ram_q			: out	std_logic_vector( 15 downto 0 );
		ram_q_en		: out	std_logic
	);
end simplesystem_ram;

architecture rtl of simplesystem_ram is
	type ram_t is array( 0 to (2**end_address_bit-1) ) of std_logic_vector( 7 downto 0 );
	signal ram_h		: ram_t;
	signal ram_l		: ram_t;
	signal index		: integer;

	signal ff_q_en		: std_logic;
	signal ff_qh		: std_logic_vector( 7 downto 0 );
	signal ff_ql		: std_logic_vector( 7 downto 0 );
begin

	-- output assignment
	ram_q		<= ff_qh & ff_ql;
	ram_q_en	<= ff_q_en;

	-- write
	index		<= conv_integer( ram_a );
	process( clk )
	begin
		if( clk'event and clk = '1' )then
			if( ram_we0 = '1' )then
				ram_l( index ) <= ram_d( 7 downto 0 );
			end if;
		end if;
	end process;

	process( clk )
	begin
		if( clk'event and clk = '1' )then
			if( ram_we1 = '1' )then
				ram_h( index ) <= ram_d( 15 downto 8 );
			end if;
		end if;
	end process;

	-- read
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_q_en		<= '0';
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_q_en		<= '0';
			else
				ff_q_en		<= ram_oe or ram_we0 or ram_we1;
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_qh		<= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_qh		<= (others => '0');
			elsif( (not ram_we1 and ram_oe) = '1' )then
				ff_qh		<= ram_h( index );
			else
				ff_qh		<= (others => '0');
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_ql		<= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_ql		<= (others => '0');
			elsif( (not ram_we0 and ram_oe) = '1' )then
				ff_ql		<= ram_l( index );
			else
				ff_ql		<= (others => '0');
			end if;
		end if;
	end process;
end rtl;
