-------------------------------------------------------------------------------
--	testbench																 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity tb is
end tb;

architecture bhv of tb is
	component simplesystem_sdcard
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;	-- 50MHz
			sreset			: in	std_logic;
			sdc_a			: in	std_logic;
			sdc_d			: in	std_logic_vector( 15 downto 0 );
			sdc_oe			: in	std_logic;
			sdc_we0			: in	std_logic;
			sdc_we1			: in	std_logic;
			sdc_q			: out	std_logic_vector( 15 downto 0 );
			sdc_q_en		: out	std_logic;
			-- to SD/MMC card (SPI mode)
			sd_clk			: out	std_logic;	-- mmc_clk
			sd_cmd			: in	std_logic;	-- mmc_di
			sd_dat3			: out	std_logic;	-- mmc_cs
			sd_dat			: out	std_logic	-- mmc_do
		);
	end component;

	signal reset			: std_logic := '1';
	signal clk				: std_logic := '0';
	signal sreset			: std_logic := '0';
	signal sdc_a			: std_logic := '0';
	signal sdc_d			: std_logic_vector( 15 downto 0 ) := (others => '0');
	signal sdc_oe			: std_logic := '0';
	signal sdc_we0			: std_logic := '0';
	signal sdc_we1			: std_logic := '0';
	signal sdc_q			: std_logic_vector( 15 downto 0 );
	signal sdc_q_en			: std_logic;
	signal sd_clk			: std_logic;
	signal sd_cmd			: std_logic := '0';
	signal sd_dat3			: std_logic;
	signal sd_dat			: std_logic;

	signal tb_count			: integer := 0;
	signal tb_end			: std_logic := '0';
begin

	-- ----------------------------------------------------- --
	--	target												 --
	-- ----------------------------------------------------- --
	u_target: simplesystem_sdcard
	port map (
		reset			=> reset			,
		clk				=> clk				,
		sreset			=> sreset			,
		sdc_a			=> sdc_a			,
		sdc_d			=> sdc_d			,
		sdc_oe			=> sdc_oe			,
		sdc_we0			=> sdc_we0			,
		sdc_we1			=> sdc_we1			,
		sdc_q			=> sdc_q			,
		sdc_q_en		=> sdc_q_en			,
		sd_clk			=> sd_clk			,
		sd_cmd			=> sd_cmd			,
		sd_dat3			=> sd_dat3			,
		sd_dat			=> sd_dat			
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
	begin
		wait until( clk'event and clk = '1' );
		reset	<= '0';

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-- cs low
		tb_count<= tb_count + 1;
		sdc_a	<= '0';
		sdc_d	<= "00000000" & "00000001";
		sdc_we0	<= '1';
		wait until( clk'event and clk = '1' );

		sdc_a	<= '0';
		sdc_d	<= "00000000" & "00000000";
		sdc_we0	<= '0';
		wait until( sdc_q_en = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-- write data
		tb_count<= tb_count + 1;
		sdc_a	<= '1';
		sdc_d	<= "00000000" & "10110101";
		sdc_we0	<= '1';
		wait until( clk'event and clk = '1' );

		sdc_a	<= '0';
		sdc_d	<= "00000000" & "00000000";
		sdc_we0	<= '0';
		wait until( sdc_q_en = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-- read data
		tb_count<= tb_count + 1;
		sdc_a	<= '1';
		sdc_oe	<= '1';
		wait until( clk'event and clk = '1' );

		sdc_a	<= '0';
		sdc_oe	<= '0';
		wait until( sdc_q_en = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		-- cs high
		tb_count<= tb_count + 1;
		sdc_a	<= '0';
		sdc_d	<= "00000000" & "00000000";
		sdc_we0	<= '1';
		wait until( clk'event and clk = '1' );

		sdc_a	<= '0';
		sdc_d	<= "00000000" & "00000000";
		sdc_we0	<= '0';
		wait until( sdc_q_en = '1' );

		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );
		wait until( clk'event and clk = '1' );

		tb_end <= '1';
		wait;
	end process;
end bhv;
