-------------------------------------------------------------------------------
--	SimpleSystem 7 segment LED												 --
--	Copyright(C)2012 t.hara													 --
-------------------------------------------------------------------------------
--	History																	 --
--	2012/08/22	t.hara	first release										 --
-------------------------------------------------------------------------------
--	Address																	 --
--	0	rw	set 16bit for output data										 --
-------------------------------------------------------------------------------
library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_seg7 is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		seg7_d			: in	std_logic_vector( 15 downto 0 );
		seg7_oe			: in	std_logic;
		seg7_we0		: in	std_logic;
		seg7_we1		: in	std_logic;
		seg7_q			: out	std_logic_vector( 15 downto 0 );
		seg7_q_en		: out	std_logic;
		seg7_out		: out	std_logic_vector( 27 downto 0 )
	);
end simplesystem_seg7;

architecture rtl of simplesystem_seg7 is
	signal ff_q			: std_logic_vector( 15 downto 0 );
	signal ff_q_en		: std_logic;
	signal ff_out		: std_logic_vector( 15 downto 0 );
	signal ff_seg7		: std_logic_vector( 27 downto 0 );
	signal w_dec0		: std_logic_vector(  6 downto 0 );
	signal w_dec1		: std_logic_vector(  6 downto 0 );
	signal w_dec2		: std_logic_vector(  6 downto 0 );
	signal w_dec3		: std_logic_vector(  6 downto 0 );
begin

	-- output assignment
	seg7_out	<= ff_seg7;
	seg7_q		<= ff_q;
	seg7_q_en	<= ff_q_en;

	-- output signal
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_out(  7 downto  0 ) <= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_out(  7 downto  0 ) <= (others => '0');
			elsif( seg7_we0 = '1' )then
				ff_out(  7 downto  0 ) <= seg7_d(  7 downto 0 );
			end if;
		end if;
	end process;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_out( 15 downto  8 ) <= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_out( 15 downto  8 ) <= (others => '0');
			elsif( seg7_we1 = '1' )then
				ff_out( 15 downto  8 ) <= seg7_d( 15 downto 8 );
			end if;
		end if;
	end process;

	-- read back
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_q	<= (others => '0');
			ff_q_en	<= '0';
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_q	<= (others => '0');
				ff_q_en	<= '0';
			elsif( (seg7_oe or seg7_we0 or seg7_we1) = '1' )then
				ff_q	<= ff_out;
				ff_q_en	<= '1';
			else
				ff_q	<= (others => '0');
				ff_q_en	<= '0';
			end if;
		end if;
	end process;

	-- 7segment decoder
	with ff_out(  3 downto  0 ) select w_dec0 <=
		"1000000"		when "0000",
		"1111001"		when "0001",
		"0100100"		when "0010",
		"0110000"		when "0011",
		"0011001"		when "0100",
		"0010010"		when "0101",
		"0000010"		when "0110",
		"1111000"		when "0111",
		"0000000"		when "1000",
		"0010000"		when "1001",
		"0001000"		when "1010",
		"0000011"		when "1011",
		"0100111"		when "1100",
		"0100001"		when "1101",
		"0000110"		when "1110",
		"0001110"		when "1111",
		(others => '0')	when others;

	with ff_out(  7 downto  4 ) select w_dec1 <=
		"1000000"		when "0000",
		"1111001"		when "0001",
		"0100100"		when "0010",
		"0110000"		when "0011",
		"0011001"		when "0100",
		"0010010"		when "0101",
		"0000010"		when "0110",
		"1111000"		when "0111",
		"0000000"		when "1000",
		"0010000"		when "1001",
		"0001000"		when "1010",
		"0000011"		when "1011",
		"0100111"		when "1100",
		"0100001"		when "1101",
		"0000110"		when "1110",
		"0001110"		when "1111",
		(others => '0')	when others;

	with ff_out( 11 downto  8 ) select w_dec2 <=
		"1000000"		when "0000",
		"1111001"		when "0001",
		"0100100"		when "0010",
		"0110000"		when "0011",
		"0011001"		when "0100",
		"0010010"		when "0101",
		"0000010"		when "0110",
		"1111000"		when "0111",
		"0000000"		when "1000",
		"0010000"		when "1001",
		"0001000"		when "1010",
		"0000011"		when "1011",
		"0100111"		when "1100",
		"0100001"		when "1101",
		"0000110"		when "1110",
		"0001110"		when "1111",
		(others => '0')	when others;

	with ff_out( 15 downto 12 ) select w_dec3 <=
		"1000000"		when "0000",
		"1111001"		when "0001",
		"0100100"		when "0010",
		"0110000"		when "0011",
		"0011001"		when "0100",
		"0010010"		when "0101",
		"0000010"		when "0110",
		"1111000"		when "0111",
		"0000000"		when "1000",
		"0010000"		when "1001",
		"0001000"		when "1010",
		"0000011"		when "1011",
		"0100111"		when "1100",
		"0100001"		when "1101",
		"0000110"		when "1110",
		"0001110"		when "1111",
		(others => '0')	when others;

	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_seg7	<= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_seg7	<= (others => '0');
			else
				ff_seg7	<= w_dec3 & w_dec2 & w_dec1 & w_dec0;
			end if;
		end if;
	end process;
end rtl;
