#!/usr/bin/perl -W
#------------------------------------------------------------------------------
#  ROM generator for smallsystem_cpu
#------------------------------------------------------------------------------
#  History
#  2012/08/20  t.hara  first release
#------------------------------------------------------------------------------
	use POSIX;		# for ceil()

	if( @ARGV < 1 ) {
		print "Usage> rom_gen.pl <image.dmp>\n";
		exit;
	}

#------------------------------------------------------------------------------
	$dest_name = $ARGV[0];
	if( $dest_name =~ /\..+$/ ) {
		$dest_name =~ s/\..+$/.vhd/;
	}
	else {
		$dest_name = $dest_name. ".vhd";
	}
	open( INFILE, $ARGV[0] ) or die( "ERROR: cannot open $ARGV[0]\n" );
	open( OUTFILE, ">$dest_name" ) or die( "ERROR: cannot create $dest_name.\n" );

#------------------------------------------------------------------------------
	my @memory_image = ();
	my $word_count = 0;
	my $address_width = 0;
	my $end_address = 0;
	while( <INFILE> ) {
		$line = $_;
		my @token = split(/ /, $line );
		shift @token;
		$word_count = $word_count + scalar( @token );
		push @memory_image, @token;
	}
	$address_width = ceil( log($word_count) / log(2) );
	$word_count = $word_count - 1;
	if( $word_count < 1 || $address_width < 1 ) {
		die( "ERROR: $ARGV[0] is short.\n" );
	}
	$end_address = (1 << $address_width) - 1;
	$address_width--;

#------------------------------------------------------------------------------
#	output ROM header

	print OUTFILE 
"-------------------------------------------------------------------------------
--	smallsystem ROM image (auto generated by rom_gen.pl)					 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_rom is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic;
		rom_a			: in	std_logic_vector( $address_width downto 0 );
		rom_oe			: in	std_logic;
		rom_q			: out	std_logic_vector( 15 downto 0 );
		rom_q_en		: out	std_logic
	);
end simplesystem_rom;

architecture rtl of simplesystem_rom is
	type rom_t is array( 0 to $end_address ) of std_logic_vector( 15 downto 0 );
	constant rom_image : rom_t :=(
";

	my $line_count = 0;
	for( $i = 0; $i <= $end_address; $i++ ) {
		if( $line_count == 0 ) {
			print OUTFILE "		";
		}
		if( $i <= $word_count ) {
			$line = $memory_image[$i];
			$line =~ s/\n//;
			print OUTFILE sprintf( "X\"%s\"", $line );
		}
		else {
			print OUTFILE "X\"0000\"";
		}
		if( $i < $end_address ) {
			print OUTFILE ", ";
		}
		if( $line_count == 7 ) {
			print OUTFILE "\n";
		}
		$line_count = ($line_count + 1) & 7;
	}

	print OUTFILE "	);

	signal ff_q_en		: std_logic;
	signal ff_q			: std_logic_vector( 15 downto 0 );
begin

	-- output assignment
	rom_q		<= ff_q;
	rom_q_en	<= ff_q_en;

	-- flip flop
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_q_en		<= '0';
			ff_q		<= (others => '0');
		elsif( clk'event and clk = '1' )then
			if( sreset = '1' )then
				ff_q_en		<= '0';
				ff_q		<= (others => '0');
			else
				ff_q_en		<= rom_oe;
				if( rom_oe = '1' )then
					ff_q		<= rom_image( conv_integer( rom_a ) );
				else
					ff_q		<= (others => '0');
				end if;
			end if;
		end if;
	end process;
end rtl;
";

	close( OUTFILE );
	close( INFILE );
	exit;
