#!/usr/bin/perl -W
#------------------------------------------------------------------------------
#  Assembler for smallsystem_cpu
#------------------------------------------------------------------------------
#  History
#  2012/08/17  t.hara  first release
#------------------------------------------------------------------------------
	use warnings;

	if( @ARGV < 1 ) {
		print "Usage> small_asm.pl <source.asm>\n";
		exit;
	}
	$debug_out = 0;

#------------------------------------------------------------------------------
	%command_list = (
		'add'		=> 0,		'sub'		=> 1,		'umul'		=> 2,		'smul'		=> 3,
		'lsft'		=> 4,		'rsft'		=> 5,		'asft'		=> 6,		'or'		=> 7,
		'xor'		=> 8,		'and'		=> 9,		'cmp'		=> 10,		'ext8'		=> 11,
		'ext16'		=> 12,		'irqret'	=> 13,		'irqen'		=> 14,		'irqvec'	=> 15,
		'ifeq'		=> 16,		'ifneq'		=> 17,		'ifgte'		=> 18,		'iflt'		=> 19,
		'uset6'		=> 20,		'sset6'		=> 21,		'load8'		=> 32,		'load16'	=> 33,
		'load32'	=> 34,		'set32'		=> 35,		'store8'	=> 48,		'store16'	=> 49,
		'store32'	=> 50
	);
	%operand_type_list = (		# 0: r1,r2,r3/imm3    1: r1    2: r3/imm3    3: r1,imm6    4: r1,imm32    5: r2,r3/imm3
		'add'		=> 0,		'sub'		=> 0,		'umul'		=> 0,		'smul'		=> 0,
		'lsft'		=> 0,		'rsft'		=> 0,		'asft'		=> 0,		'or'		=> 0,
		'xor'		=> 0,		'and'		=> 0,		'cmp'		=> 0,		'ext8'		=> 0,
		'ext16'		=> 0,		'irqret'	=> 1,		'irqen'		=> 2,		'irqvec'	=> 5,
		'ifeq'		=> 0,		'ifneq'		=> 0,		'ifgte'		=> 0,		'iflt'		=> 0,
		'uset6'		=> 3,		'sset6'		=> 3,		'load8'		=> 0,		'load16'	=> 0,
		'load32'	=> 0,		'set32'		=> 4,		'store8'	=> 0,		'store16'	=> 0,
		'store32'	=> 0
	);
	%register_list = (
		'r0'		=> 0,		'r1'		=> 1,		'r2'		=> 2,		'r3'		=> 3,
		'r4'		=> 4,		'r5'		=> 5,		'r6'		=> 6,		'r7'		=> 7,
		'pc'		=> 7
	);

#------------------------------------------------------------------------------
	$dest_name = $ARGV[0];
	if( $dest_name =~ /\..+$/ ) {
		$dest_name =~ s/\..+$/.dmp/;
	}
	else {
		$dest_name = $dest_name. ".dmp";
	}
	open( OUTFILE, ">$dest_name" ) or die( "ERROR: cannot create $dest_name.\n" );

#------------------------------------------------------------------------------
	my %symbol;
	my $name;

	foreach $path ( 0..1 ) {
		print "path ". ($path + 1). "\n";
		$current_address	 = 0;
		foreach $name ( @ARGV ) {
			&do_compile( $name );
		}
	}

	close( OUTFILE );
	exit;

#------------------------------------------------------------------------------
sub my_eval {
	my $eval_value = 0;

	# replace symbols
	$line = ' '. $_[0]. ' ';
	$line =~ s/\$/$current_address/g;
	foreach $name ( keys( %symbol ) ) {
		$line =~ s/(.*[ \t\+\-\*\/\(\)\&|\%])$name([ \t\+\-\*\/\(\)\&\|\%].*)/$1$symbol{$name}$2/g;
	}
	$eval_value = eval( $line );
	if( $@ ) {
		printf( "%s(%d) : %s\n", $source_name, $line_no, $@ );
		$eval_value = 0;
	}
	return $eval_value;
}

#------------------------------------------------------------------------------
sub do_compile {

	open( INFILE, $_[0] ) or die( "ERROR: cannot read $_[0]\n" );
	print "compile $_[0]\n";

	$source_name = $_[0];
	$line_no = 1;
	while( $line = <INFILE> ) {
		&line_compile( $line );
		$line_no = $line_no + 1;
	}
	close( INFILE );
}

#------------------------------------------------------------------------------
sub process_command {
	@immediate_mode_str = ( '.r', '.i' );

	my $command_name = $_[0];
	my $operand = $_[2];
	if( !exists($command_list{$command_name}) ) {
		printf( "%s(%d) : invalid command (%s)\n", $source_name, $line_no, $command_name );
		return;
	}
	my $immediate_mode = 0;
	if( !defined($_[1]) ) {
		$immediate_mode = 0;
	}
	elsif( $_[1] eq '' || $_[1] eq '.r' ) {
		$immediate_mode = 0;
	}
	elsif( $_[1] eq '.i' ) {
		$immediate_mode = 1;
	}
	else {
		printf( "%s(%d) : invalid postfix (%s)\n", $source_name, $line_no, $_[1] );
		return;
	}
	my $operand_type = $operand_type_list{$command_name};
	my $destination_register = 0;
	my $source1_register = 0;
	my $source2_register = 0;
	my $imm3 = 0;
	my $imm6 = 0;
	my $imm32 = 0;
	# 1st operand
	if( $operand_type == 0 || $operand_type == 1 || $operand_type == 3 || $operand_type == 4 ) {
		if( $operand =~ /\s*([a-z0-9]+)/ ) {
			if( !defined($register_list{$1}) ) {
				printf( "%s(%d) : illegal operand (%s)\n", $source_name, $line_no, $1 );
				return;
			}
			$destination_register = $register_list{$1};
			$operand =~ s/\s*[a-z0-9]+\s*\,?//;
		}
		else {
			printf( "%s(%d) : operand not found\n", $source_name, $line_no );
			return;
		}
	}
	# 2nd operand
	if( $operand_type == 0 || $operand_type == 5 ) {
		if( $operand =~ /\s*([a-z0-9]+)/ ) {
			if( !defined($register_list{$1}) ) {
				printf( "%s(%d) : illegal operand (%s)\n", $source_name, $line_no, $1 );
				return;
			}
			$source1_register = $register_list{$1};
			$operand =~ s/\s*[a-z0-9]+\s*\,?//;
		}
		else {
			printf( "%s(%d) : operand not found\n", $source_name, $line_no );
			return;
		}
	}
	# 3rd operand
	if( ($operand_type == 0 || $operand_type == 2 || $operand_type == 5) && $immediate_mode == 0 ) {
		if( $operand =~ /\s*([a-z0-9]+)/ ) {
			if( !defined($register_list{$1}) ) {
				printf( "%s(%d) : illegal operand (%s)\n", $source_name, $line_no, $1 );
				return;
			}
			$source2_register = $register_list{$1};
			$operand =~ s/\s*[a-z0-9]+\s*\,?//;
		}
		else {
			printf( "%s(%d) : operand not found\n", $source_name, $line_no );
			return;
		}
	}
	if( $path == 1 ) {
		# imm3 operand
		if( ($operand_type == 0 || $operand_type == 2 || $operand_type == 5) && $immediate_mode == 1 ) {
			$imm3 = &my_eval( $operand );
			if( $imm3 < 0 || $imm3 > 7 ) {
				printf( "%s(%d) : illegal range (%d)\n", $source_name, $line_no, $imm3 );
				return;
			}
			$source2_register = $imm3;
		}
		# imm6 operand
		if( $operand_type == 3 ) {
			$imm6 = &my_eval( $operand );
			if( $imm6 < -32 || $imm6 > 31 ) {
				printf( "%s(%d) : illegal range (%d)\n", $source_name, $line_no, $imm3 );
				return;
			}
			$source2_register = $imm6;
		}
		# imm32 operand
		if( $operand_type == 4 ) {
			$imm32 = &my_eval( $operand );
		}
	}
	# output image
	my $output = sprintf( "%08X %04X", $current_address, ($immediate_mode << 15) | ($command_list{$command_name} << 9) | ($destination_register << 6) | ($source1_register << 3) | $source2_register );
	if( $operand_type == 4 ) {
		$output = $output. sprintf( " %04X %04X", $imm32 & 0xFFFF, ($imm32 >> 16) & 0xFFFF );
		$current_address = $current_address + 6;
	}
	else {
		$current_address = $current_address + 2;
	}
	if( $debug_out ) {
		print "found [$_[0]$immediate_mode_str[$immediate_mode]] : $output\n";
	}
	if( $path == 1 ) {
		print OUTFILE $output. "\n";
	}
}

#------------------------------------------------------------------------------
sub string_dump {
	my $str = $_[0];

	while( $str ne "" ) {
		push( @byte_dump, ord($str) );
		$str = substr( $str, 1 );
	}
}

#------------------------------------------------------------------------------
sub process_db {
	my $line = $_[0];
	my $str;

	@byte_dump = ();
	$line =~ s/.*\.db//;
	while( $line ne "" ) {
		$line =~ s/^\s*//;
		$str = $line;
		if( substr( $line, 0, 1 ) eq '"' ) {
			$str =~ s/"(.*?)"/$1/;
			&string_dump( $str );
		}
		else {
			if( $path == 1 ) {
				$str =~ s/,.*//;
				push( @byte_dump, &my_eval($str) & 255 );
			}
			else {
				push( @byte_dump, 0 );
			}
		}
		if( $line =~ /,/ ) {
			$line =~ s/.*?,//;
		}
		else {
			$line = "";
		}
	}
	# padding
	if( (scalar( @byte_dump ) & 1) == 1 ) {
		push( @byte_dump, 0 );
	}
	# output
	my $output = sprintf( "%08X", $current_address );
	my $value;
	while( scalar( @byte_dump ) > 0 ) {
		$value = shift( @byte_dump ) & 255;
		$value = $value | ((shift( @byte_dump ) & 255) << 8);
		$output = $output. sprintf( " %04X", $value );
		$current_address = $current_address + 2;
	}
	if( $path == 1 ) {
		print OUTFILE $output. "\n";
	}
}

#------------------------------------------------------------------------------
sub process_dw {
	my $line = $_[0];
	my $output = sprintf( "%08X", $current_address );
	my $value = 0;
	my $str;

	$line =~ s/.*\.dw//;
	while( $line ne "" ) {
		$line =~ s/^\s*//;
		$str = $line;
		if( $path == 1 ) {
			$str =~ s/,.*//;
			$value = &my_eval($str) & 65535;
		}
		$output = $output. sprintf( " %04X", $value );
		$current_address = $current_address + 2;
		if( $line =~ /,/ ) {
			$line =~ s/.*?,//;
		}
		else {
			$line = "";
		}
	}
	if( $path == 1 ) {
		print OUTFILE $output. "\n";
	}
}

#------------------------------------------------------------------------------
sub process_dd {
	my $line = $_[0];
	my $output = sprintf( "%08X", $current_address );
	my $value = 0;
	my $str;

	$line =~ s/.*\.dd//;
	while( $line ne "" ) {
		$line =~ s/^\s*//;
		$str = $line;
		if( $path == 1 ) {
			$str =~ s/,.*//;
			$value = &my_eval($str);
		}
		$output = $output. sprintf( " %04X %04X", $value & 65535, ($value >> 16) & 65535 );
		$current_address = $current_address + 4;
		if( $line =~ /,/ ) {
			$line =~ s/.*?,//;
		}
		else {
			$line = "";
		}
	}
	if( $path == 1 ) {
		print OUTFILE $output. "\n";
	}
}

#------------------------------------------------------------------------------
sub line_compile {

	my $line = $_[0];
	# delete comment
	$line =~ s/;.*//;
	$line =~ s/\n//;
	$line = lc( $line );
	if( $debug_out ) {
		print "[$line]\n";
	}
	# process .org
	if( $line =~ /\.org\s(.*)/ ) {
		$current_address = &my_eval( $1 );
		if( $path == 0 && ($current_address & 1) == 1 ) {
			printf( "%s(%d) : .org odd address (0x%08X)\n", $source_name, $line_no, $current_address );
		}
		$current_address = $current_address & ~1;
		if( $debug_out ) {
			print "found [.org] ==> current_address = $current_address\n";
		}
		$line =~ s/\.org\s(.*)//;
	}
	# process .db
	if( $line =~ /\.db\s(.*)/ ) {
		&process_db( $line );
		$line =~ s/\.db\s(.*)//;
	}
	# process .dw
	if( $line =~ /\.dw\s(.*)/ ) {
		&process_dw( $line );
		$line =~ s/\.dw\s(.*)//;
	}
	# process .dd
	if( $line =~ /\.dd\s(.*)/ ) {
		&process_dd( $line );
		$line =~ s/\.dd\s(.*)//;
	}
	# process label
	if( $line =~ /\s*([a-z0-9_]+)\s*:/ ) {
		if( $debug_out ) {
			print "found label ==> $1 = $current_address\n";
		}
		if( $path == 0 && defined( $symbol{ $1 } ) ) {
			printf( "%s(%d) : multiple defined label (%s)\n", $source_name, $line_no, $1 );
		}
		else {
			$symbol{ $1 } = $current_address;
		}
		$line =~ s/\s*[a-z0-9_]+\s*://;
	}
	# process symbol
	if( $line =~ /\s*([a-z0-9_]+)\s*\=(.*)/ ) {
		if( $path == 1 ) {
			if( $debug_out ) {
				print "found symbol ==> $1 = ". &my_eval( $2 ). "\n";
			}
			$symbol{ $1 } = &my_eval( $2 );
		}
		return;
	}
	# process command
	if( $line =~ /\s*([a-z0-9]+)(\.?[ri]?)\s*(.*)/ ) {
		&process_command( $1, $2, $3 );
	}
}
