-------------------------------------------------------------------------------
--	smallsystem top entity (auto generated by top_gen.pl)					 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_top is
	port(
		nreset			: in	std_logic;
		clk				: in	std_logic;
		pio_pio_in		: in	std_logic_vector( 15 downto 0 );
		pio_pio_out	: out	std_logic_vector( 15 downto 0 )
	);
end simplesystem_top;

architecture rtl of simplesystem_top is

	component simplesystem_rom
		port(
			nreset			: in	std_logic;
			clk				: in	std_logic;
			rom_a			: in	std_logic_vector( 6 downto 0 );
			rom_oe			: in	std_logic;
			rom_q			: out	std_logic_vector( 15 downto 0 );
			rom_q_en		: out	std_logic
		);
	end component;

	component simplesystem_ram
		generic(
			end_address_bit	: in	integer := 5		-- (2^n - 1)
		);
		port(
			nreset			: in	std_logic;
			clk				: in	std_logic;
			ram_a			: in	std_logic_vector( end_address_bit-1 downto 0 );
			ram_d			: in	std_logic_vector( 15 downto 0 );
			ram_oe			: in	std_logic;
			ram_we0			: in	std_logic;
			ram_we1			: in	std_logic;
			ram_q			: out	std_logic_vector( 15 downto 0 );
			ram_q_en		: out	std_logic
		);
	end simplesystem_ram;

	component simplesystem_timer
		port(
			nreset			: in	std_logic;
			clk				: in	std_logic;
			timer_a			: in	std_logic_vector(  1 downto 0 );
			timer_d			: in	std_logic_vector( 15 downto 0 );
			timer_oe		: in	std_logic;
			timer_we0		: in	std_logic;
			timer_we1		: in	std_logic;
			timer_q			: out	std_logic_vector( 15 downto 0 );
			timer_q_en		: out	std_logic;
			timer_irq		: out	std_logic
		);
	end component;

	component simplesystem_pio
		port(
			nreset			: in	std_logic;
			clk				: in	std_logic;
			pio_a			: in	std_logic;
			pio_d			: in	std_logic_vector( 15 downto 0 );
			pio_oe			: in	std_logic;
			pio_we0			: in	std_logic;
			pio_we1			: in	std_logic;
			pio_q			: out	std_logic_vector( 15 downto 0 );
			pio_q_en		: out	std_logic;
			pio_in			: in	std_logic_vector( 15 downto 0 );
			pio_out			: out	std_logic_vector( 15 downto 0 )
		);
	end component;

	component simplesystem_cpu
		port(
			nreset			: in	std_logic;
			clk				: in	std_logic;
			-- interrupt request
			irq				: in	std_logic;
			-- bus interface
			cpu_address		: out	std_logic_vector( 30 downto 0 );
			cpu_d			: out	std_logic_vector( 15 downto 0 );
			cpu_we0			: out	std_logic;							-- lower byte
			cpu_we1			: out	std_logic;							-- upper byte
			cpu_oe			: out	std_logic;
			cpu_q			: in	std_logic_vector( 15 downto 0 );
			cpu_q_en		: in	std_logic
		);
	end component;
	signal rom_rom_a			: std_logic_vector( 6 downto 0 );
	signal rom_rom_oe			: std_logic;
	signal rom_rom_q			: std_logic_vector( 15 downto 0 );
	signal rom_rom_q_en		: std_logic;
	signal rom_rom_en			: std_logic;
	signal ram_ram_a		: std_logic_vector( 7-1 downto 0 );
	signal ram_ram_d		: std_logic_vector( 15 downto 0 );
	signal ram_ram_oe		: std_logic;
	signal ram_ram_we0		: std_logic;
	signal ram_ram_we1		: std_logic;
	signal ram_ram_q		: std_logic_vector( 15 downto 0 );
	signal ram_ram_q_en	: std_logic;
	signal ram_ram_en		: std_logic;
	signal timer1_timer_a			: std_logic_vector(  1 downto 0 );
	signal timer1_timer_d			: std_logic_vector( 15 downto 0 );
	signal timer1_timer_oe		: std_logic;
	signal timer1_timer_we0		: std_logic;
	signal timer1_timer_we1		: std_logic;
	signal timer1_timer_q			: std_logic_vector( 15 downto 0 );
	signal timer1_timer_q_en		: std_logic;
	signal timer1_timer_irq		: std_logic;
	signal timer1_timer_en		: std_logic;
	signal timer2_timer_a			: std_logic_vector(  1 downto 0 );
	signal timer2_timer_d			: std_logic_vector( 15 downto 0 );
	signal timer2_timer_oe		: std_logic;
	signal timer2_timer_we0		: std_logic;
	signal timer2_timer_we1		: std_logic;
	signal timer2_timer_q			: std_logic_vector( 15 downto 0 );
	signal timer2_timer_q_en		: std_logic;
	signal timer2_timer_irq		: std_logic;
	signal timer2_timer_en		: std_logic;
	signal pio_pio_a			: std_logic;
	signal pio_pio_d			: std_logic_vector( 15 downto 0 );
	signal pio_pio_oe			: std_logic;
	signal pio_pio_we0			: std_logic;
	signal pio_pio_we1			: std_logic;
	signal pio_pio_q			: std_logic_vector( 15 downto 0 );
	signal pio_pio_q_en		: std_logic;
	signal pio_pio_en			: std_logic;

	signal irq				: std_logic;
	signal cpu_address		: std_logic_vector( 30 downto 0 );
	signal cpu_d			: std_logic_vector( 15 downto 0 );
	signal cpu_we0			: std_logic;
	signal cpu_we1			: std_logic;
	signal cpu_oe			: std_logic;
	signal cpu_q			: std_logic_vector( 15 downto 0 );
	signal cpu_q_en			: std_logic;
begin

	rom_rom_a	<=	cpu_a(6 downto 0);

	u_rom: simplesystem_rom
	port map(
		nreset			=> nreset,
		clk				=> clk,
		rom_a			=> rom_rom_a,
		rom_oe			=> rom_rom_oe,
		rom_q			=> rom_rom_q,
		rom_q_en		=> rom_rom_q_en
	);

	ram_ram_a	<=	cpu_a(7-1 downto 0);
	ram_ram_d	<=	cpu_d;

	u_ram: simplesystem_ram
	generic map(
		end_address_bit	=> 7
	);
	port map(
		nreset			=> nreset,
		clk				=> clk,
		ram_a			=> ram_ram_a,
		ram_d			=> ram_ram_d,
		ram_oe			=> ram_ram_oe,
		ram_we0			=> ram_ram_we0,
		ram_we1			=> ram_ram_we1,
		ram_q			=> ram_ram_q,
		ram_q_en		=> ram_ram_q_en
	);

	timer1_timer_a	<=	cpu_a(1 downto 0);
	timer1_timer_d	<=	cpu_d;

	u_timer1: simplesystem_timer
	port map(
		nreset			=> nreset,
		clk				=> clk,
		timer_a			=> timer1_timer_a,
		timer_d			=> timer1_timer_d,
		timer_oe		=> timer1_timer_oe,
		timer_we0		=> timer1_timer_we0,
		timer_we1		=> timer1_timer_we1,
		timer_q			=> timer1_timer_q,
		timer_q_en		=> timer1_timer_q_en,
		timer_irq		=> timer1_timer_irq
	);

	timer2_timer_a	<=	cpu_a(1 downto 0);
	timer2_timer_d	<=	cpu_d;

	u_timer2: simplesystem_timer
	port map(
		nreset			=> nreset,
		clk				=> clk,
		timer_a			=> timer2_timer_a,
		timer_d			=> timer2_timer_d,
		timer_oe		=> timer2_timer_oe,
		timer_we0		=> timer2_timer_we0,
		timer_we1		=> timer2_timer_we1,
		timer_q			=> timer2_timer_q,
		timer_q_en		=> timer2_timer_q_en,
		timer_irq		=> timer2_timer_irq
	);

	pio_pio_a	<=	cpu_a(0);
	pio_pio_d	<=	cpu_d;

	u_pio: simplesystem_pio
	port map(
		nreset			=> nreset,
		clk				=> clk,
		pio_a			=> pio_pio_a,
		pio_d			=> pio_pio_d,
		pio_oe			=> pio_pio_oe,
		pio_we0			=> pio_pio_we0,
		pio_we1			=> pio_pio_we1,
		pio_q			=> pio_pio_q,
		pio_q_en		=> pio_pio_q_en,
		pio_in			=> pio_pio_in,
		pio_out			=> pio_pio_out
	);
	rom_rom_en		=>	'1'		when( cpu_a(30 downto 6 ) = "0000000000000000000000000" )else '0';
	rom_rom_oe		=>	cpu_oe	when( rom_rom_en = '1' )else '0';

	ram_ram_en		=>	'1'		when( cpu_a(30 downto 7 ) = "000000000000000000000001" )else '0';
	ram_ram_oe		=>	cpu_oe	when( ram_ram_en = '1' )else '0';
	ram_ram_we0	=>	cpu_we0	when( ram_ram_en = '1' )else '0';
	ram_ram_we1	=>	cpu_we1	when( ram_ram_en = '1' )else '0';

	timer1_timer_en		=>	'1'		when( cpu_a(30 downto 2 ) = "0000000000000000010000000000" )else '0';
	timer1_timer_oe	=>	cpu_oe	when( timer1_timer_en = '1' )else '0';
	timer1_timer_we0	=>	cpu_we0	when( timer1_timer_en = '1' )else '0';
	timer1_timer_we1	=>	cpu_we1	when( timer1_timer_en = '1' )else '0';

	timer2_timer_en		=>	'1'		when( cpu_a(30 downto 2 ) = "0000000000000000100000000000" )else '0';
	timer2_timer_oe	=>	cpu_oe	when( timer2_timer_en = '1' )else '0';
	timer2_timer_we0	=>	cpu_we0	when( timer2_timer_en = '1' )else '0';
	timer2_timer_we1	=>	cpu_we1	when( timer2_timer_en = '1' )else '0';

	pio_pio_en		=>	'1'		when( cpu_a(30 downto 1 ) = "00000000000000001100000000000" )else '0';
	pio_pio_oe		=>	cpu_oe	when( pio_pio_en = '1' )else '0';
	pio_pio_we0	=>	cpu_we0	when( pio_pio_en = '1' )else '0';
	pio_pio_we1	=>	cpu_we1	when( pio_pio_en = '1' )else '0';


	cpu_q		<= rom_rom_q    or ram_ram_q    or timer1_timer_q    or timer2_timer_q    or pio_pio_q   ;
	cpu_q_en	<= rom_rom_q_en or ram_ram_q_en or timer1_timer_q_en or timer2_timer_q_en or pio_pio_q_en;

	irq <=	timer1_timer_irq or timer2_timer_irq;

	u_cpu: simplesystem_cpu
	port map(
		nreset			=> nreset,
		clk				=> clk,
		irq				=> irq,
		cpu_address		=> cpu_address, 
		cpu_d			=> cpu_d, 
		cpu_we0			=> cpu_we0, 
		cpu_we1			=> cpu_we1, 
		cpu_oe			=> cpu_oe, 
		cpu_q			=> cpu_q, 
		cpu_q_en		=> cpu_q_en 
	);

end rtl;
