#!/usr/bin/perl -W
#------------------------------------------------------------------------------
#  TOP generator for smallsystem
#------------------------------------------------------------------------------
#  History
#  2012/08/24  t.hara  first release
#------------------------------------------------------------------------------
	use POSIX;		# for ceil()

	if( @ARGV < 1 ) {
		print "Usage> top_gen.pl <config.txt>\n";
		exit;
	}

#------------------------------------------------------------------------------
	open( OUTFILE, ">top.vhd" ) or die( "ERROR: cannot create top.vhd\n" );

#------------------------------------------------------------------------------
	my @entity = ();
	my @component = ();
	my @added_component = ();
	my @signal = ();
	my @instance = ();
	my @address_list = ();
	my @irq_list = ();
	my @data_q_list = ();

#------------------------------------------------------------------------------
#	makeup header
#------------------------------------------------------------------------------
	print OUTFILE 
"-------------------------------------------------------------------------------
--	smallsystem top entity (auto generated by top_gen.pl)					 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity simplesystem_top is
	port(
		reset			: in	std_logic;
		clk				: in	std_logic;
		sreset			: in	std_logic";

#------------------------------------------------------------------------------
#	configration
#------------------------------------------------------------------------------
	open( INFILE, $ARGV[0] ) or die( "ERROR: cannot open $ARGV[0]\n" );

	$line_no = 1;
	while( <INFILE> ) {
		&config_process( $_ );
		$line_no = $line_no + 1;
	}

	close( INFILE );

	# port --------------------------------------------------------------------
	foreach $line ( @entity ) {
		print OUTFILE ";\n\t\t". $line;
	}
	print OUTFILE "
	);
end simplesystem_top;

architecture rtl of simplesystem_top is
";

	# components --------------------------------------------------------------
	foreach $line ( @component ) {
		print OUTFILE $line;
	}
	print OUTFILE "
	component simplesystem_cpu
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			-- interrupt request
			irq				: in	std_logic;
			-- bus interface
			cpu_address		: out	std_logic_vector( 30 downto 0 );
			cpu_d			: out	std_logic_vector( 15 downto 0 );
			cpu_we0			: out	std_logic;							-- lower byte
			cpu_we1			: out	std_logic;							-- upper byte
			cpu_oe			: out	std_logic;
			cpu_q			: in	std_logic_vector( 15 downto 0 );
			cpu_q_en		: in	std_logic
		);
	end component;
";

	# signals -----------------------------------------------------------------
	foreach $line ( @signal ) {
		print OUTFILE $line. "\n";
	}
	print OUTFILE "
	signal irq				: std_logic;
	signal cpu_address		: std_logic_vector( 30 downto 0 );
	signal cpu_d			: std_logic_vector( 15 downto 0 );
	signal cpu_we0			: std_logic;
	signal cpu_we1			: std_logic;
	signal cpu_oe			: std_logic;
	signal cpu_q			: std_logic_vector( 15 downto 0 );
	signal cpu_q_en			: std_logic;
begin
";

	# instance ----------------------------------------------------------------
	foreach $line ( @instance ) {
		print OUTFILE $line;
	}

	# address decoder ---------------------------------------------------------
	&address_decoder_generate();

	# cpu ---------------------------------------------------------------------
	if( scalar( @irq_list ) == 0 ) {
		print OUTFILE "
	irq <= '0';
";
	}
	else {
		print OUTFILE "
	irq <=	";

		my $or_split = "";
		foreach $line ( @irq_list ) {
			print OUTFILE $or_split. $line;
			$or_split = " or ";
		}
		print OUTFILE ";\n";
	}

	print OUTFILE "
	u_cpu: simplesystem_cpu
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		irq				=> irq,
		cpu_address		=> cpu_address, 
		cpu_d			=> cpu_d, 
		cpu_we0			=> cpu_we0, 
		cpu_we1			=> cpu_we1, 
		cpu_oe			=> cpu_oe, 
		cpu_q			=> cpu_q, 
		cpu_q_en		=> cpu_q_en 
	);
";

	# footer ------------------------------------------------------------------
	print OUTFILE "
end rtl;
";
	close( OUTFILE );

#------------------------------------------------------------------------------
#	testbench
	open( OUTFILE, ">tb.vhd" ) or die( "ERROR: cannot create tb.vhd\n" );

	print OUTFILE 
"-------------------------------------------------------------------------------
--	smallsystem top entity (auto generated by top_gen.pl)					 --
-------------------------------------------------------------------------------

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_signed.all;
	use	ieee.std_logic_arith.all;

entity tb is
end tb;

architecture bhv of tb is
	component simplesystem_top
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic";

	foreach $line ( @entity ) {
		print OUTFILE ";\n\t\t\t". $line;
	}
	print OUTFILE "
		);
	end component;

	signal reset			: std_logic := '1';
	signal clk				: std_logic := '0';
	signal sreset			: std_logic := '0';
";

	foreach $line ( @entity ) {
		$line =~s/;//;
		$line =~s/: in\s+/: /;
		$line =~s/: out\s+/: /;
		$line =~s/: inout\s+/: /;
		print OUTFILE "\tsignal ". $line. ";\n";
	}

	print OUTFILE "
	signal tb_end			: std_logic := '0';
begin

	-- ----------------------------------------------------- --
	--	target												 --
	-- ----------------------------------------------------- --
	u_target: simplesystem_top
	port map (
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset";

	foreach $line ( @entity ) {
		$line =~s/;//;
		$line =~s/:.*//;
		print OUTFILE ",\n\t\t". $line. " => ". $line;
	}

	print OUTFILE "
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	-----------------------------------------------------------
	--	test bench											 --
	-----------------------------------------------------------
	process
	begin
		wait until( clk'event and clk = '1' );
		reset	<= '0';

		for i in 0 to 1000 loop
			wait until( clk'event and clk = '1' );
		end loop;
		tb_end <= '1';
		wait;
	end process;
end bhv;
";

	close( OUTFILE );
	exit;

#------------------------------------------------------------------------------
sub config_process {

	my $line = $_[0];

	$line =~s/^\s*//;		# delete white space on line top
	$line =~s/;.*$//;		# delete comment
	$line = lc( $line );

	my @token = split(/\s+/, $line );
	if( scalar( @token ) < 1 ) {
		return;
	}

	if(		$token[0] eq "pio"		) { &pio_generate( @token ); }
	elsif(	$token[0] eq "timer"	) { &timer_generate( @token ); }
	elsif(	$token[0] eq "rom"		) { &rom_generate( @token ); }
	elsif(	$token[0] eq "ram"		) { &ram_generate( @token ); }
	elsif(	$token[0] eq "seg7"		) { &seg7_generate( @token ); }
	elsif(	$token[0] eq "sram"		) { &sram_generate( @token ); }
	else {
		print "ERROR: unsupported device \"$token[0]\" at $line_no.\n";
	}
}

#------------------------------------------------------------------------------
sub address_decoder_generate {

	foreach $line ( @address_list ) {
		print OUTFILE $line. "\n";
	}

	print OUTFILE "
	cpu_q		<= ";
	my $or_split = "";
	foreach $line ( @data_q_list ) {
		print OUTFILE $or_split. $line. "_q   ";
		$or_split = " or ";
	}
	print OUTFILE ";";

	print OUTFILE "
	cpu_q_en	<= ";
	$or_split = "";
	foreach $line ( @data_q_list ) {
		print OUTFILE $or_split. $line. "_q_en";
		$or_split = " or ";
	}
	print OUTFILE ";\n";
}

#------------------------------------------------------------------------------
sub pio_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);

	print "PIO [${instance_name}] : ". sprintf( "0x%08X", $address ). "\n";

	if( grep(/pio/, @added_component ) == 0 ) {
		push( @added_component, "pio" );
		push( @component, "
	component simplesystem_pio
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			pio_a			: in	std_logic;
			pio_d			: in	std_logic_vector( 15 downto 0 );
			pio_oe			: in	std_logic;
			pio_we0			: in	std_logic;
			pio_we1			: in	std_logic;
			pio_q			: out	std_logic_vector( 15 downto 0 );
			pio_q_en		: out	std_logic;
			pio_in			: in	std_logic_vector( 15 downto 0 );
			pio_out			: out	std_logic_vector( 15 downto 0 )
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_pio_a	<=	cpu_address(0);
	${instance_name}_pio_d	<=	cpu_d;

	u_$instance_name: simplesystem_pio
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		pio_a			=> ${instance_name}_pio_a,
		pio_d			=> ${instance_name}_pio_d,
		pio_oe			=> ${instance_name}_pio_oe,
		pio_we0			=> ${instance_name}_pio_we0,
		pio_we1			=> ${instance_name}_pio_we1,
		pio_q			=> ${instance_name}_pio_q,
		pio_q_en		=> ${instance_name}_pio_q_en,
		pio_in			=> ${instance_name}_pio_in,
		pio_out			=> ${instance_name}_pio_out
	);
" );

	push( @entity, "${instance_name}_pio_in		: in	std_logic_vector( 15 downto 0 )" );
	push( @entity, "${instance_name}_pio_out	: out	std_logic_vector( 15 downto 0 )" );

	push( @signal, "	signal ${instance_name}_pio_a			: std_logic;" );
	push( @signal, "	signal ${instance_name}_pio_d			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_pio_oe			: std_logic;" );
	push( @signal, "	signal ${instance_name}_pio_we0			: std_logic;" );
	push( @signal, "	signal ${instance_name}_pio_we1			: std_logic;" );
	push( @signal, "	signal ${instance_name}_pio_q			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_pio_q_en		: std_logic;" );
	push( @signal, "	signal ${instance_name}_pio_en			: std_logic;" );

	my $address_bit = make_bin( $address >> (1+1), 30 - 1 );
	push( @address_list, "	${instance_name}_pio_en		<=	'1'		when( cpu_address(30 downto 1 ) = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_pio_oe		<=	cpu_oe	when( ${instance_name}_pio_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_pio_we0	<=	cpu_we0	when( ${instance_name}_pio_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_pio_we1	<=	cpu_we1	when( ${instance_name}_pio_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_pio" );
}

#------------------------------------------------------------------------------
sub timer_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);

	print "TIMER [${instance_name}] : ". sprintf( "0x%08X", $address ). "\n";

	if( grep(/timer/, @added_component ) == 0 ) {
		push( @added_component, "timer" );
		push( @component, "
	component simplesystem_timer
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			timer_a			: in	std_logic_vector(  1 downto 0 );
			timer_d			: in	std_logic_vector( 15 downto 0 );
			timer_oe		: in	std_logic;
			timer_we0		: in	std_logic;
			timer_we1		: in	std_logic;
			timer_q			: out	std_logic_vector( 15 downto 0 );
			timer_q_en		: out	std_logic;
			timer_irq		: out	std_logic
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_timer_a	<=	cpu_address(1 downto 0);
	${instance_name}_timer_d	<=	cpu_d;

	u_$instance_name: simplesystem_timer
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		timer_a			=> ${instance_name}_timer_a,
		timer_d			=> ${instance_name}_timer_d,
		timer_oe		=> ${instance_name}_timer_oe,
		timer_we0		=> ${instance_name}_timer_we0,
		timer_we1		=> ${instance_name}_timer_we1,
		timer_q			=> ${instance_name}_timer_q,
		timer_q_en		=> ${instance_name}_timer_q_en,
		timer_irq		=> ${instance_name}_timer_irq
	);
" );

	push( @signal, "	signal ${instance_name}_timer_a			: std_logic_vector(  1 downto 0 );" );
	push( @signal, "	signal ${instance_name}_timer_d			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_timer_oe		: std_logic;" );
	push( @signal, "	signal ${instance_name}_timer_we0		: std_logic;" );
	push( @signal, "	signal ${instance_name}_timer_we1		: std_logic;" );
	push( @signal, "	signal ${instance_name}_timer_q			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_timer_q_en		: std_logic;" );
	push( @signal, "	signal ${instance_name}_timer_irq		: std_logic;" );
	push( @signal, "	signal ${instance_name}_timer_en		: std_logic;" );

	my $address_bit = make_bin( $address >> (2+1), 30 - 2 );
	push( @address_list, "	${instance_name}_timer_en  <= '1'     when( cpu_address(30 downto 2 ) = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_timer_oe  <= cpu_oe  when( ${instance_name}_timer_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_timer_we0 <= cpu_we0 when( ${instance_name}_timer_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_timer_we1 <= cpu_we1 when( ${instance_name}_timer_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_timer" );

	push( @irq_list, "${instance_name}_timer_irq" );
}

#------------------------------------------------------------------------------
sub rom_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);
	my $address_size = eval($_[3]) | 1;

	print "ROM [${instance_name}] : ". sprintf( "0x%08X-%08X", $address, $address + $address_size ). "\n";

	$address_size = $address_size >> 1;

	my $address_high = ceil( log( $address_size ) / log(2) ) - 1;

	if( grep(/rom$address_high/, @added_component ) == 0 ) {
		push( @added_component, "rom$address_high" );
		push( @component, "
	component simplesystem_rom
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			rom_a			: in	std_logic_vector( ${address_high} downto 0 );
			rom_oe			: in	std_logic;
			rom_q			: out	std_logic_vector( 15 downto 0 );
			rom_q_en		: out	std_logic
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_rom_a	<=	cpu_address(${address_high} downto 0);

	u_$instance_name: simplesystem_rom
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		rom_a			=> ${instance_name}_rom_a,
		rom_oe			=> ${instance_name}_rom_oe,
		rom_q			=> ${instance_name}_rom_q,
		rom_q_en		=> ${instance_name}_rom_q_en
	);
" );

	push( @signal, "	signal ${instance_name}_rom_a			: std_logic_vector( ${address_high} downto 0 );" );
	push( @signal, "	signal ${instance_name}_rom_oe			: std_logic;" );
	push( @signal, "	signal ${instance_name}_rom_q			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_rom_q_en		: std_logic;" );
	push( @signal, "	signal ${instance_name}_rom_en			: std_logic;" );

	my $address_bit = make_bin( $address >> (${address_high} + 1), 30 - (${address_high}+1) + 1 );
	push( @address_list, "	${instance_name}_rom_en		<=	'1'		when( cpu_address(30 downto ${address_high}+1 ) = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_rom_oe		<=	cpu_oe	when( ${instance_name}_rom_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_rom" );
}

#------------------------------------------------------------------------------
sub ram_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);
	my $address_size = eval($_[3]) | 1;

	print "RAM [${instance_name}] : ". sprintf( "0x%08X-%08X", $address, $address + $address_size ). "\n";

	$address_size = $address_size >> 1;

	my $address_high = ceil( log( $address_size ) / log(2) );

	if( grep(/ram/, @added_component ) == 0 ) {
		push( @added_component, "ram" );
		push( @component, "
	component simplesystem_ram
		generic(
			end_address_bit	: in	integer := 5		-- (2^n - 1)
		);
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			ram_a			: in	std_logic_vector( end_address_bit-1 downto 0 );
			ram_d			: in	std_logic_vector( 15 downto 0 );
			ram_oe			: in	std_logic;
			ram_we0			: in	std_logic;
			ram_we1			: in	std_logic;
			ram_q			: out	std_logic_vector( 15 downto 0 );
			ram_q_en		: out	std_logic
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_ram_a	<=	cpu_address(${address_high}-1 downto 0);
	${instance_name}_ram_d	<=	cpu_d;

	u_$instance_name: simplesystem_ram
	generic map(
		end_address_bit	=> ${address_high}
	)
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		ram_a			=> ${instance_name}_ram_a,
		ram_d			=> ${instance_name}_ram_d,
		ram_oe			=> ${instance_name}_ram_oe,
		ram_we0			=> ${instance_name}_ram_we0,
		ram_we1			=> ${instance_name}_ram_we1,
		ram_q			=> ${instance_name}_ram_q,
		ram_q_en		=> ${instance_name}_ram_q_en
	);
" );

	push( @signal, "	signal ${instance_name}_ram_a		: std_logic_vector( ${address_high}-1 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_d		: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_oe		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_we0		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_we1		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_q		: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_q_en	: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_en		: std_logic;" );

	my $address_bit = make_bin( $address >> (${address_high} + 1), 30 - ${address_high} + 1 );
	push( @address_list, "	${instance_name}_ram_en		<=	'1'		when( cpu_address(30 downto ${address_high} ) = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_ram_oe		<=	cpu_oe	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_ram_we0	<=	cpu_we0	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_ram_we1	<=	cpu_we1	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_ram" );
}

#------------------------------------------------------------------------------
sub seg7_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);

	print "SEG7 [${instance_name}] : ". sprintf( "0x%08X", $address ). "\n";

	if( grep(/seg7/, @added_component ) == 0 ) {
		push( @added_component, "seg7" );
		push( @component, "
	component simplesystem_seg7
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			seg7_d			: in	std_logic_vector( 15 downto 0 );
			seg7_oe			: in	std_logic;
			seg7_we0		: in	std_logic;
			seg7_we1		: in	std_logic;
			seg7_q			: out	std_logic_vector( 15 downto 0 );
			seg7_q_en		: out	std_logic;
			seg7_out		: out	std_logic_vector( 27 downto 0 )
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_seg7_d	<=	cpu_d;

	u_$instance_name: simplesystem_seg7
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		seg7_d			=> ${instance_name}_seg7_d,
		seg7_oe			=> ${instance_name}_seg7_oe,
		seg7_we0		=> ${instance_name}_seg7_we0,
		seg7_we1		=> ${instance_name}_seg7_we1,
		seg7_q			=> ${instance_name}_seg7_q,
		seg7_q_en		=> ${instance_name}_seg7_q_en,
		seg7_out		=> ${instance_name}_seg7_out
	);
" );

	push( @entity, "${instance_name}_seg7_out	: out	std_logic_vector( 27 downto 0 )" );

	push( @signal, "	signal ${instance_name}_seg7_d			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_seg7_oe			: std_logic;" );
	push( @signal, "	signal ${instance_name}_seg7_we0		: std_logic;" );
	push( @signal, "	signal ${instance_name}_seg7_we1		: std_logic;" );
	push( @signal, "	signal ${instance_name}_seg7_q			: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_seg7_q_en		: std_logic;" );
	push( @signal, "	signal ${instance_name}_seg7_en			: std_logic;" );

	my $address_bit = make_bin( $address >> 1, 31 );
	push( @address_list, "	${instance_name}_seg7_en	<=	'1'		when( cpu_address = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_seg7_oe	<=	cpu_oe	when( ${instance_name}_seg7_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_seg7_we0	<=	cpu_we0	when( ${instance_name}_seg7_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_seg7_we1	<=	cpu_we1	when( ${instance_name}_seg7_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_seg7" );
}

#------------------------------------------------------------------------------
sub sram_generate {

	my $instance_name = $_[1];
	my $address = eval($_[2]);

	print "RAM [${instance_name}] : ". sprintf( "0x%08X-%08X", $address, $address + 512*1024 - 1 ). "\n";

	if( grep(/sram/, @added_component ) == 0 ) {
		push( @added_component, "sram" );
		push( @component, "
	component simplesystem_sram
		port(
			reset			: in	std_logic;
			clk				: in	std_logic;
			sreset			: in	std_logic;
			ram_a			: in	std_logic_vector( 17 downto 0 );
			ram_d			: in	std_logic_vector( 15 downto 0 );
			ram_oe			: in	std_logic;
			ram_we0			: in	std_logic;
			ram_we1			: in	std_logic;
			ram_q			: out	std_logic_vector( 15 downto 0 );
			ram_q_en		: out	std_logic;
			-- to SRAM
			sram_addr		: out	std_logic_vector( 17 downto 0 );
			sram_dq			: inout	std_logic_vector( 15 downto 0 );
			sram_we_n		: out	std_logic;
			sram_oe_n		: out	std_logic;
			sram_ub_n		: out	std_logic;
			sram_lb_n		: out	std_logic;
			sram_ce_n		: out	std_logic
		);
	end component;
" );
	}

	push( @instance, "
	${instance_name}_ram_a	<=	cpu_address(17 downto 0);
	${instance_name}_ram_d	<=	cpu_d;

	u_$instance_name: simplesystem_sram
	port map(
		reset			=> reset,
		clk				=> clk,
		sreset			=> sreset,
		ram_a			=> ${instance_name}_ram_a,
		ram_d			=> ${instance_name}_ram_d,
		ram_oe			=> ${instance_name}_ram_oe,
		ram_we0			=> ${instance_name}_ram_we0,
		ram_we1			=> ${instance_name}_ram_we1,
		ram_q			=> ${instance_name}_ram_q,
		ram_q_en		=> ${instance_name}_ram_q_en,
		sram_addr		=> ${instance_name}_sram_addr,
		sram_dq			=> ${instance_name}_sram_dq,
		sram_we_n		=> ${instance_name}_sram_we_n,
		sram_oe_n		=> ${instance_name}_sram_oe_n,
		sram_ub_n		=> ${instance_name}_sram_ub_n,
		sram_lb_n		=> ${instance_name}_sram_lb_n,
		sram_ce_n		=> ${instance_name}_sram_ce_n
	);
" );

	push( @entity, "${instance_name}_sram_addr	: out	std_logic_vector( 17 downto 0 )" );
	push( @entity, "${instance_name}_sram_dq	: inout	std_logic_vector( 15 downto 0 )" );
	push( @entity, "${instance_name}_sram_we_n	: out	std_logic" );
	push( @entity, "${instance_name}_sram_oe_n	: out	std_logic" );
	push( @entity, "${instance_name}_sram_ub_n	: out	std_logic" );
	push( @entity, "${instance_name}_sram_lb_n	: out	std_logic" );
	push( @entity, "${instance_name}_sram_ce_n	: out	std_logic" );

	push( @signal, "	signal ${instance_name}_ram_a		: std_logic_vector( 17 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_d		: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_oe		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_we0		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_we1		: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_q		: std_logic_vector( 15 downto 0 );" );
	push( @signal, "	signal ${instance_name}_ram_q_en	: std_logic;" );
	push( @signal, "	signal ${instance_name}_ram_en		: std_logic;" );

	my $address_bit = make_bin( $address >> (18 + 1), 30 - 18 + 1 );
	push( @address_list, "	${instance_name}_ram_en		<=	'1'		when( cpu_address(30 downto 18 ) = ${address_bit} )else '0';" );
	push( @address_list, "	${instance_name}_ram_oe		<=	cpu_oe	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_ram_we0	<=	cpu_we0	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "	${instance_name}_ram_we1	<=	cpu_we1	when( ${instance_name}_ram_en = '1' )else '0';" );
	push( @address_list, "" );

	push( @data_q_list, "${instance_name}_ram" );
}

#------------------------------------------------------------------------------
sub make_bin {

	my $address = $_[0];
	my $count = $_[1];

	my $output = "";
	for( $i = 0; $i < $count; $i++ ) {
		if( ($address & 1) == 0 ) {
			$output = "0". $output;
		}
		else {
			$output = "1". $output;
		}
		$address = $address >> 1;
	}
	$output = '"'. $output. '"';
	return $output;
}
